@@ -23,15 +23,16 @@ import kotlinx.serialization.json.Json
 import org.slf4j.Logger
 import org.slf4j.LoggerFactory
 import tech.libeufin.common.*
+import tech.libeufin.common.db.DatabaseConfig
 import java.nio.file.Path
+import java.time.Duration
 
 private val logger: Logger = LoggerFactory.getLogger("libeufin-bank")
 
-/**
- * Application the parsed configuration.
- */
+/** Configuration for libeufin-bank */
 data class BankConfig(
     val name: String,
+    val baseUrl: String?,
     val regionalCurrency: String,
     val regionalCurrencySpec: CurrencySpecification,
     val allowRegistration: Boolean,
@@ -47,7 +48,10 @@ data class BankConfig(
     val spaPath: Path?,
     val tanChannels: Map<TanChannel, Pair<Path, Map<String, String>>>,
     val payto: BankPaytoCtx,
-    val wireMethod: WireMethod
+    val wireMethod: WireMethod,
+    val gcAbortAfter: Duration,
+    val gcCleanAfter: Duration,
+    val gcDeleteAfter: Duration
 )
 
 @Serializable
@@ -132,16 +136,20 @@ fun TalerConfig.loadBankConfig(): BankConfig {
         allowAccountDeletion = lookupBoolean("libeufin-bank", "allow_account_deletion") ?: false,
         allowEditName = lookupBoolean("libeufin-bank", "allow_edit_name") ?: false,
         allowEditCashout = lookupBoolean("libeufin-bank", "allow_edit_cashout_payto_uri") ?: false,
+        allowConversion = allowConversion,
         defaultDebtLimit = amount("libeufin-bank", "default_debt_limit", regionalCurrency) ?: TalerAmount(0, 0, regionalCurrency),
         registrationBonus = amount("libeufin-bank", "registration_bonus", regionalCurrency) ?: TalerAmount(0, 0, regionalCurrency),
         suggestedWithdrawalExchange = lookupString("libeufin-bank", "suggested_withdrawal_exchange"),
         spaPath = lookupPath("libeufin-bank", "spa"),
-        allowConversion = allowConversion,
+        baseUrl = lookupString("libeufin-bank", "base_url"),
         fiatCurrency = fiatCurrency,
         fiatCurrencySpec = fiatCurrencySpec,
         tanChannels = tanChannels,
         payto = payto,
-        wireMethod = method
+        wireMethod = method,
+        gcAbortAfter = requireDuration("libeufin-bank", "gc_abort_after"),
+        gcCleanAfter = requireDuration("libeufin-bank", "gc_clean_after"),
+        gcDeleteAfter = requireDuration("libeufin-bank", "gc_delete_after"),
     )
 }
 
