@@ -35,12 +35,6 @@ import io.ktor.server.application.*
 import io.ktor.server.engine.*
 import io.ktor.server.http.content.*
 import io.ktor.server.netty.*
-import io.ktor.server.plugins.*
-import io.ktor.server.plugins.callloging.*
-import io.ktor.server.plugins.contentnegotiation.*
-import io.ktor.server.plugins.cors.routing.*
-import io.ktor.server.plugins.forwardedheaders.*
-import io.ktor.server.plugins.statuspages.*
 import io.ktor.server.request.*
 import io.ktor.server.response.*
 import io.ktor.server.routing.*
@@ -51,207 +45,47 @@ import org.postgresql.util.PSQLState
 import org.slf4j.Logger
 import org.slf4j.LoggerFactory
 import org.slf4j.event.Level
+import tech.libeufin.bank.api.*
 import tech.libeufin.bank.db.AccountDAO.*
 import tech.libeufin.bank.db.Database
 import tech.libeufin.common.*
+import tech.libeufin.common.api.*
+import tech.libeufin.common.db.dbInit
+import tech.libeufin.common.db.pgDataSource
 import java.net.InetAddress
 import java.sql.SQLException
+import java.time.Instant
 import java.util.zip.DataFormatException
 import java.util.zip.Inflater
 import kotlin.io.path.Path
 import kotlin.io.path.exists
 import kotlin.io.path.readText
-import tech.libeufin.bank.api.*
 
 private val logger: Logger = LoggerFactory.getLogger("libeufin-bank")
 // Dirty local variable to stop the server in test TODO remove this ugly hack
-var engine: ApplicationEngine? = null 
+var engine: ApplicationEngine? = null
 
-/**
- * This plugin checks for body length limit and inflates the requests that have "Content-Encoding: deflate"
- */
-val bodyPlugin = createApplicationPlugin("BodyLimitAndDecompression") {
-    onCallReceive { call ->
-        // TODO check content length as an optimisation
-        transformBody { body ->
-            val bytes = ByteArray(MAX_BODY_LENGTH.toInt() + 1)
-            var read = 0
-            when (val encoding = call.request.headers[HttpHeaders.ContentEncoding])  {
-                "deflate" -> {
-                    // Decompress and check decompressed length
-                    val inflater = Inflater()
-                    while (!body.isClosedForRead) {
-                        body.read { buf ->
-                            inflater.setInput(buf)
-                            try {
-                                read += inflater.inflate(bytes, read, bytes.size - read)
-                            } catch (e: DataFormatException) {
-                                logger.error("Deflated request failed to inflate: ${e.message}")
-                                throw badRequest(
-                                    "Could not inflate request",
-                                    TalerErrorCode.GENERIC_COMPRESSION_INVALID
-                                )
-                            }
-                        }
-                        if (read > MAX_BODY_LENGTH)
-                            throw badRequest("Decompressed body is suspiciously big > $MAX_BODY_LENGTH B")
-                    }
-                }
-                null -> {
-                    // Check body length
-                    while (true) {
-                        val new = body.readAvailable(bytes, read, bytes.size - read)
-                        if (new == -1) break // Channel is closed
-                        read += new
-                        if (read > MAX_BODY_LENGTH)
-                            throw badRequest("Body is suspiciously big > $MAX_BODY_LENGTH B")
-                    }
-                } 
-                else -> throw unsupportedMediaType(
-                    "Content encoding '$encoding' not supported, expected plain or deflate",
-                    TalerErrorCode.GENERIC_COMPRESSION_INVALID
-                )
-            }
-            ByteReadChannel(bytes, 0, read)
-        }
-    }
-}
 
 /**
  * Set up web server handlers for the Taler corebank API.
  */
-fun Application.corebankWebApp(db: Database, ctx: BankConfig) {
-    install(CallLogging) {
-        this.level = Level.INFO
-        this.logger = tech.libeufin.bank.logger
-        this.format { call ->
-            val status = call.response.status()
-            val httpMethod = call.request.httpMethod.value
-            val path = call.request.path()
-            val msg = call.logMsg()
-            if (msg != null) {
-                "$status, $httpMethod $path, $msg"
-            } else {
-                "$status, $httpMethod $path"
-            }
-        }
-    }
-    install(XForwardedHeaders)
-    install(CORS) {
-        anyHost()
-        allowHeader(HttpHeaders.Authorization)
-        allowHeader(HttpHeaders.ContentType)
-        allowMethod(HttpMethod.Options)
-        allowMethod(HttpMethod.Patch)
-        allowMethod(HttpMethod.Delete)
-        allowCredentials = true
-    }
-    install(bodyPlugin)
-    install(IgnoreTrailingSlash)
-    install(ContentNegotiation) {
-        json(Json {
-            @OptIn(ExperimentalSerializationApi::class)
-            explicitNulls = false
-            encodeDefaults = true
-            ignoreUnknownKeys = true
-        })
-    }
-    install(StatusPages) {
-        status(HttpStatusCode.NotFound) { call, status ->
-            call.err(
-                status,
-                "There is no endpoint defined for the URL provided by the client. Check if you used the correct URL and/or file a report with the developers of the client software.",
-                TalerErrorCode.GENERIC_ENDPOINT_UNKNOWN
-            )
-        }
-        status(HttpStatusCode.MethodNotAllowed) { call, status ->
-            call.err(
-                status,
-                "The HTTP method used is invalid for this endpoint. This is likely a bug in the client implementation. Check if you are using the latest available version and/or file a report with the developers.",
-                TalerErrorCode.GENERIC_METHOD_INVALID
-            )
-        }
-        exception<Exception> { call, cause ->
-            logger.debug("request failed", cause)
-            when (cause) {
-                is LibeufinException -> call.err(cause)
-                is SQLException -> {
-                    when (cause.sqlState) {
-                        PSQLState.SERIALIZATION_FAILURE.state -> call.err(
-                            HttpStatusCode.InternalServerError,
-                            "Transaction serialization failure",
-                            TalerErrorCode.BANK_SOFT_EXCEPTION
-                        )
-                        else -> call.err(
-                            HttpStatusCode.InternalServerError,
-                            "Unexpected sql error with state ${cause.sqlState}",
-                            TalerErrorCode.BANK_UNMANAGED_EXCEPTION
-                        )
-                    }
-                }
-                is BadRequestException -> {
-                    /**
-                     * NOTE: extracting the root cause helps with JSON error messages,
-                     * because they mention the particular way they are invalid, but OTOH
-                     * it loses (by getting null) other error messages, like for example
-                     * the one from MissingRequestParameterException.  Therefore, in order
-                     * to get the most detailed message, we must consider BOTH sides:
-                     * the 'cause' AND its root cause!
-                     */
-                    var rootCause: Throwable? = cause.cause
-                    while (rootCause?.cause != null)
-                        rootCause = rootCause.cause
-                    // Telling apart invalid JSON vs missing parameter vs invalid parameter.
-                    val talerErrorCode = when {
-                        cause is MissingRequestParameterException ->
-                            TalerErrorCode.GENERIC_PARAMETER_MISSING
-                        cause is ParameterConversionException ->
-                            TalerErrorCode.GENERIC_PARAMETER_MALFORMED
-                        rootCause is CommonError -> when (rootCause) {
-                            is CommonError.AmountFormat -> TalerErrorCode.BANK_BAD_FORMAT_AMOUNT
-                            is CommonError.AmountNumberTooBig -> TalerErrorCode.BANK_NUMBER_TOO_BIG
-                            is CommonError.Payto -> TalerErrorCode.GENERIC_JSON_INVALID
-                        }
-                        else -> TalerErrorCode.GENERIC_JSON_INVALID
-                    }
-                    call.err(
-                        badRequest(
-                            cause.message,
-                            talerErrorCode,
-                            /* Here getting _some_ error message, by giving precedence
-                            * to the root cause, as otherwise JSON details would be lost. */
-                            rootCause?.message
-                        )
-                    )
-                }
-                else -> {
-                    call.err(
-                        HttpStatusCode.InternalServerError,
-                        cause.message,
-                        TalerErrorCode.BANK_UNMANAGED_EXCEPTION
-                    )
-                }
-            }
-        }
-    }
-    routing {
-        coreBankApi(db, ctx)
-        conversionApi(db, ctx)
-        bankIntegrationApi(db, ctx)
-        wireGatewayApi(db, ctx)
-        revenueApi(db, ctx)
-        ctx.spaPath?.let {
-            get("/") {
-                call.respondRedirect("/webui/")
-            }
-            staticFiles("/webui/", it.toFile())
+fun Application.corebankWebApp(db: Database, ctx: BankConfig) = talerApi(logger) {
+    coreBankApi(db, ctx)
+    conversionApi(db, ctx)
+    bankIntegrationApi(db, ctx)
+    wireGatewayApi(db, ctx)
+    revenueApi(db, ctx)
+    ctx.spaPath?.let {
+        get("/") {
+            call.respondRedirect("/webui/")
         }
+        staticFiles("/webui/", it.toFile())
     }
 }
 
 class BankDbInit : CliktCommand("Initialize the libeufin-bank database", name = "dbinit") {
     private val common by CommonOption()
-    private val requestReset by option(
+    private val reset by option(
         "--reset", "-r",
         help = "Reset database (DANGEROUS: All existing data is lost)"
     ).flag()
@@ -260,13 +94,8 @@ class BankDbInit : CliktCommand("Initialize the libeufin-bank database", name =
         val config = talerConfig(common.config)
         val cfg = config.loadDbConfig()
         val ctx = config.loadBankConfig()
-        Database(cfg.dbConnStr, ctx.regionalCurrency, ctx.fiatCurrency).use { db -> 
-            db.conn { conn ->
-                if (requestReset) {
-                    resetDatabaseTables(conn, cfg, sqlFilePrefix = "libeufin-bank")
-                }
-                initializeDatabaseTables(conn, cfg, sqlFilePrefix = "libeufin-bank")
-            }
+        pgDataSource(cfg.dbConnStr).dbInit(cfg, "libeufin-bank", reset)
+        Database(cfg, ctx.regionalCurrency, ctx.fiatCurrency).use { db ->
             // Create admin account if missing
             val res = createAdminAccount(db, ctx) // logs provided by the helper
             when (res) {
@@ -274,7 +103,7 @@ class BankDbInit : CliktCommand("Initialize the libeufin-bank database", name =
                 AccountCreationResult.LoginReuse -> {}
                 AccountCreationResult.PayToReuse -> 
                     throw Exception("Failed to create admin's account")
-                AccountCreationResult.Success ->
+                is AccountCreationResult.Success ->
                     logger.info("Admin's account created")
             }
         }
@@ -289,7 +118,7 @@ class ServeBank : CliktCommand("Run libeufin-bank HTTP server", name = "serve")
         val ctx = cfg.loadBankConfig()
         val dbCfg = cfg.loadDbConfig()
         val serverCfg = cfg.loadServerConfig()
-        Database(dbCfg.dbConnStr, ctx.regionalCurrency, ctx.fiatCurrency).use { db ->
+        Database(dbCfg, ctx.regionalCurrency, ctx.fiatCurrency).use { db ->
             if (ctx.allowConversion) {
                 logger.info("Ensure exchange account exists")
                 val info = db.account.bankInfo("exchange", ctx.payto)
@@ -348,7 +177,7 @@ class ChangePw : CliktCommand("Change account password", name = "passwd") {
         val cfg = talerConfig(common.config)
         val ctx = cfg.loadBankConfig() 
         val dbCfg = cfg.loadDbConfig()
-        Database(dbCfg.dbConnStr, ctx.regionalCurrency, ctx.fiatCurrency).use { db ->
+        Database(dbCfg, ctx.regionalCurrency, ctx.fiatCurrency).use { db ->
             val res = db.account.reconfigPassword(username, password, null, true)
             when (res) {
                 AccountPatchAuthResult.UnknownAccount ->
@@ -392,7 +221,7 @@ class EditAccount : CliktCommand(
         val cfg = talerConfig(common.config)
         val ctx = cfg.loadBankConfig() 
         val dbCfg = cfg.loadDbConfig()
-        Database(dbCfg.dbConnStr, ctx.regionalCurrency, ctx.fiatCurrency).use { db ->
+        Database(dbCfg, ctx.regionalCurrency, ctx.fiatCurrency).use { db ->
             val req = AccountReconfiguration(
                 name = name,
                 is_taler_exchange = exchange,
@@ -469,7 +298,7 @@ class CreateAccount : CliktCommand(
         val ctx = cfg.loadBankConfig() 
         val dbCfg = cfg.loadDbConfig()
 
-        Database(dbCfg.dbConnStr, ctx.regionalCurrency, ctx.fiatCurrency).use { db ->
+        Database(dbCfg, ctx.regionalCurrency, ctx.fiatCurrency).use { db ->
             val req = json ?: options?.run {
                 RegisterAccountRequest(
                     username = username,
@@ -487,27 +316,45 @@ class CreateAccount : CliktCommand(
                 ) 
             }
             req?.let {
-                val (result, internalPayto) = createAccount(db, ctx, req, true)
-                when (result) {
+                when (val result = createAccount(db, ctx, req, true)) {
                     AccountCreationResult.BonusBalanceInsufficient ->
                         throw Exception("Insufficient admin funds to grant bonus")
                     AccountCreationResult.LoginReuse ->
                         throw Exception("Account username reuse '${req.username}'")
                     AccountCreationResult.PayToReuse ->
-                        throw Exception("Bank internalPayToUri reuse '$internalPayto'")
-                    AccountCreationResult.Success ->
+                        throw Exception("Bank internalPayToUri reuse")
+                    is AccountCreationResult.Success -> {
                         logger.info("Account '${req.username}' created")
+                        println(result.payto)
+                    }
                 }
-                println(internalPayto)
             }
         }
     }
 }
 
+class GC : CliktCommand(
+    "Run garbage collection: abort expired operations and clean expired data",
+    name = "gc"
+) {
+    private val common by CommonOption()
+ 
+    override fun run() = cliCmd(logger, common.log) {
+        val cfg = talerConfig(common.config)
+        val ctx = cfg.loadBankConfig() 
+        val dbCfg = cfg.loadDbConfig()
+
+        Database(dbCfg, ctx.regionalCurrency, ctx.fiatCurrency).use { db ->
+            logger.info("Run garbage collection")
+            db.gc.collect(Instant.now(), ctx.gcAbortAfter, ctx.gcCleanAfter, ctx.gcDeleteAfter)
+        }
+    }
+}
+
 class LibeufinBankCommand : CliktCommand() {
     init {
         versionOption(getVersion())
-        subcommands(ServeBank(), BankDbInit(), CreateAccount(), EditAccount(), ChangePw(), CliConfigCmd(BANK_CONFIG_SOURCE))
+        subcommands(ServeBank(), BankDbInit(), CreateAccount(), EditAccount(), ChangePw(), GC(), CliConfigCmd(BANK_CONFIG_SOURCE))
     }
 
     override fun run() = Unit
