@@ -55,6 +55,11 @@ enum class WithdrawalStatus {
     confirmed
 }
 
+enum class AccountStatus {
+    active,
+    deleted
+}
+
 enum class RoundingMode {
     zero,
     up,
@@ -161,7 +166,7 @@ data class ChallengeContactData(
     }
     companion object {
         private val EMAIL_PATTERN = Regex("[a-zA-Z0-9._%+-]+@[a-zA-Z0-9.-]+\\.[a-zA-Z]{2,4}")
-        private val PHONE_PATTERN = Regex("^\\+?[0-9]+$")
+        private val PHONE_PATTERN = Regex("\\+?[0-9]+")
     }
 }
 
@@ -178,7 +183,16 @@ data class RegisterAccountRequest(
     val payto_uri: Payto? = null,
     val debit_threshold: TalerAmount? = null,
     val tan_channel: TanChannel? = null,
-)
+) {
+    init {
+        if (!USERNAME_REGEX.matches(username))
+            throw badRequest("username '$username' is malformed")
+    }
+
+    companion object {
+        private val USERNAME_REGEX = Regex("[a-zA-Z0-9\\-\\._~]+")
+    }
+}
 
 @Serializable
 data class RegisterAccountResponse(
@@ -277,6 +291,7 @@ data class BearerToken(
 data class Config(
     val currency: String,
     val currency_specification: CurrencySpecification,
+    val base_url: String?,
     val bank_name: String,
     val allow_conversion: Boolean,
     val allow_registrations: Boolean,
@@ -312,14 +327,6 @@ data class TalerIntegrationConfigResponse(
     val version: String = INTEGRATION_API_VERSION
 }
 
-@Serializable
-data class WireGatewayConfig(
-    val currency: String
-) {
-    val name: String = "taler-wire-gateway"
-    val version: String = WIRE_GATEWAY_API_VERSION
-}
-
 @Serializable
 data class RevenueConfig(
     val currency: String
@@ -350,7 +357,8 @@ data class AccountMinimalData(
     val debit_threshold: TalerAmount,
     val is_public: Boolean,
     val is_taler_exchange: Boolean,
-    val row_id: Long
+    val row_id: Long,
+    val status: AccountStatus
 )
 
 /**
@@ -374,13 +382,15 @@ data class AccountData(
     val cashout_payto_uri: String? = null,
     val tan_channel: TanChannel? = null,
     val is_public: Boolean,
-    val is_taler_exchange: Boolean
+    val is_taler_exchange: Boolean,
+    val status: AccountStatus
 )
 
 @Serializable
 data class TransactionCreateRequest(
     val payto_uri: Payto,
-    val amount: TalerAmount?
+    val amount: TalerAmount?,
+    val request_uid: ShortHashCode?
 )
 
 @Serializable
@@ -536,70 +546,6 @@ data class ConversionResponse(
     val amount_credit: TalerAmount,
 )
 
-/**
- * Request to an /admin/add-incoming request from
- * the Taler Wire Gateway API.
- */
-@Serializable
-data class AddIncomingRequest(
-    val amount: TalerAmount,
-    val reserve_pub: EddsaPublicKey,
-    val debit_account: Payto
-)
-
-/**
- * Response to /admin/add-incoming
- */
-@Serializable
-data class AddIncomingResponse(
-    val timestamp: TalerProtocolTimestamp,
-    val row_id: Long
-)
-
-/**
- * Response of a TWG /history/incoming call.
- */
-@Serializable
-data class IncomingHistory(
-    val incoming_transactions: List<IncomingReserveTransaction>,
-    val credit_account: String
-)
-
-/**
- * TWG's incoming payment record.
- */
-@Serializable
-data class IncomingReserveTransaction(
-    val type: String = "RESERVE",
-    val row_id: Long, // DB row ID of the payment.
-    val date: TalerProtocolTimestamp,
-    val amount: TalerAmount,
-    val debit_account: String,
-    val reserve_pub: EddsaPublicKey
-)
-
-/**
- * Response of a TWG /history/outgoing call.
- */
-@Serializable
-data class OutgoingHistory(
-    val outgoing_transactions: List<OutgoingTransaction>,
-    val debit_account: String
-)
-
-/**
- * TWG's outgoinf payment record.
- */
-@Serializable
-data class OutgoingTransaction(
-    val row_id: Long, // DB row ID of the payment.
-    val date: TalerProtocolTimestamp,
-    val amount: TalerAmount,
-    val credit_account: String,
-    val wtid: ShortHashCode,
-    val exchange_base_url: String,
-)
-
 @Serializable
 data class RevenueIncomingHistory(
     val incoming_transactions : List<RevenueIncomingBankTransaction>,
@@ -615,27 +561,6 @@ data class RevenueIncomingBankTransaction(
     val subject: String
 )
 
-/**
- * TWG's request to pay a merchant.
- */
-@Serializable
-data class TransferRequest(
-    val request_uid: HashCode,
-    val amount: TalerAmount,
-    val exchange_base_url: ExchangeUrl,
-    val wtid: ShortHashCode,
-    val credit_account: Payto
-)
-
-/**
- * TWG's response to merchant payouts
- */
-@Serializable
-data class TransferResponse(
-    val timestamp: TalerProtocolTimestamp,
-    val row_id: Long
-)
-
 /**
  * Response to GET /public-accounts
  */
