@@ -28,6 +28,7 @@ import kotlinx.coroutines.future.await
 import kotlinx.coroutines.withContext
 import org.slf4j.Logger
 import org.slf4j.LoggerFactory
+import tech.libeufin.bank.*
 import tech.libeufin.bank.auth.*
 import tech.libeufin.bank.db.AbortResult
 import tech.libeufin.bank.db.AccountDAO.*
@@ -39,7 +40,6 @@ import tech.libeufin.bank.db.TransactionDAO.BankTransactionResult
 import tech.libeufin.bank.db.WithdrawalDAO.WithdrawalConfirmationResult
 import tech.libeufin.bank.db.WithdrawalDAO.WithdrawalCreationResult
 import tech.libeufin.common.*
-import tech.libeufin.bank.*
 import java.time.Duration
 import java.time.Instant
 import java.time.temporal.ChronoUnit
@@ -52,6 +52,7 @@ fun Routing.coreBankApi(db: Database, ctx: BankConfig) {
         call.respond(
             Config(
                 bank_name = ctx.name,
+                base_url = ctx.baseUrl,
                 currency = ctx.regionalCurrency,
                 currency_specification = ctx.regionalCurrencySpec,
                 allow_conversion = ctx.allowConversion,
@@ -145,7 +146,7 @@ suspend fun createAccount(
     cfg: BankConfig, 
     req: RegisterAccountRequest, 
     isAdmin: Boolean
-): Pair<AccountCreationResult, String>  {
+): AccountCreationResult  {
     // Prohibit reserved usernames:
     if (RESERVED_ACCOUNTS.contains(req.username))
         throw conflict(
@@ -209,14 +210,15 @@ suspend fun createAccount(
                     bonus = if (!req.is_taler_exchange) cfg.registrationBonus 
                             else TalerAmount(0, 0, cfg.regionalCurrency),
                     tanChannel = req.tan_channel,
-                    checkPaytoIdempotent = req.payto_uri != null
+                    checkPaytoIdempotent = req.payto_uri != null,
+                    ctx = cfg.payto
                 )
                 // Retry with new IBAN
                 if (res == AccountCreationResult.PayToReuse && retry > 0) {
                     retry--
                     continue
                 }
-                return Pair(res, internalPayto.bank(req.name, cfg.payto))
+                return res
             }
         }
         WireMethod.X_TALER_BANK -> {
@@ -229,7 +231,7 @@ suspend fun createAccount(
          
             val internalPayto = XTalerBankPayto.forUsername(req.username)
         
-            val res = db.account.create(
+            return db.account.create(
                 login = req.username,
                 name = req.name,
                 email = req.contact_data?.email?.get(),
@@ -243,9 +245,9 @@ suspend fun createAccount(
                 bonus = if (!req.is_taler_exchange) cfg.registrationBonus 
                         else TalerAmount(0, 0, cfg.regionalCurrency),
                 tanChannel = req.tan_channel,
-                checkPaytoIdempotent = req.payto_uri != null
+                checkPaytoIdempotent = req.payto_uri != null,
+                ctx = cfg.payto
             )
-            return Pair(res, internalPayto.bank(req.name, cfg.payto))
         }
     }
 }
@@ -294,8 +296,7 @@ private fun Routing.coreBankAccountsApi(db: Database, ctx: BankConfig) {
     authAdmin(db, TokenScope.readwrite, !ctx.allowRegistration) {
         post("/accounts") {
             val req = call.receive<RegisterAccountRequest>()
-            val (result, internalPayto) = createAccount(db, ctx, req, isAdmin)
-            when (result) {
+            when (val result = createAccount(db, ctx, req, isAdmin)) {
                 AccountCreationResult.BonusBalanceInsufficient -> throw conflict(
                     "Insufficient admin funds to grant bonus",
                     TalerErrorCode.BANK_UNALLOWED_DEBIT
@@ -305,10 +306,10 @@ private fun Routing.coreBankAccountsApi(db: Database, ctx: BankConfig) {
                     TalerErrorCode.BANK_REGISTER_USERNAME_REUSE
                 )
                 AccountCreationResult.PayToReuse -> throw conflict(
-                    "Bank internalPayToUri reuse '$internalPayto'",
+                    "Bank internalPayToUri reuse",
                     TalerErrorCode.BANK_REGISTER_PAYTO_URI_REUSE
                 )
-                AccountCreationResult.Success -> call.respond(RegisterAccountResponse(internalPayto))
+                is AccountCreationResult.Success -> call.respond(RegisterAccountResponse(result.payto))
             }
         }
     }
@@ -421,7 +422,7 @@ private fun Routing.coreBankAccountsApi(db: Database, ctx: BankConfig) {
 }
 
 private fun Routing.coreBankTransactionsApi(db: Database, ctx: BankConfig) {
-    auth(db, TokenScope.readonly) {
+    auth(db, TokenScope.readonly, allowAdmin = true) {
         get("/accounts/{USERNAME}/transactions") {
             val params = HistoryParams.extract(call.request.queryParameters)
             val bankAccount = call.bankInfo(db, ctx.payto)
@@ -458,6 +459,7 @@ private fun Routing.coreBankTransactionsApi(db: Database, ctx: BankConfig) {
                 subject = subject,
                 amount = amount,
                 timestamp = Instant.now(),
+                requestUid = req.request_uid,
                 is2fa = challenge != null
             )
             when (res) {
@@ -478,6 +480,10 @@ private fun Routing.coreBankTransactionsApi(db: Database, ctx: BankConfig) {
                     "Insufficient funds",
                     TalerErrorCode.BANK_UNALLOWED_DEBIT
                 )
+                BankTransactionResult.RequestUidReuse -> throw conflict(
+                    "request_uid used already",
+                    TalerErrorCode.BANK_TRANSFER_REQUEST_UID_REUSED
+                )
                 is BankTransactionResult.Success -> call.respond(TransactionCreateResponse(res.id))
             }
         }
@@ -612,7 +618,7 @@ private fun Routing.coreBankCashoutApi(db: Database, ctx: BankConfig) = conditio
             }
         }
     }
-    auth(db, TokenScope.readonly) {
+    auth(db, TokenScope.readonly, allowAdmin = true) {
         get("/accounts/{USERNAME}/cashouts/{CASHOUT_ID}") {
             val id = call.longPath("CASHOUT_ID")
             val cashout = db.cashout.get(id, username) ?: throw notFound(
@@ -693,7 +699,7 @@ private fun Routing.coreBankTanApi(db: Database, ctx: BankConfig) {
                             exitValue
                         }
                         if (exitValue != 0) {
-                            throw libeufinError(
+                            throw apiError(
                                 HttpStatusCode.BadGateway,
                                 "Tan channel script failure with exit value $exitValue",
                                 TalerErrorCode.BANK_TAN_CHANNEL_SCRIPT_FAILED
@@ -727,7 +733,7 @@ private fun Routing.coreBankTanApi(db: Database, ctx: BankConfig) {
                     "Incorrect TAN code",
                     TalerErrorCode.BANK_TAN_CHALLENGE_FAILED
                 )
-                TanSolveResult.NoRetry -> throw libeufinError(
+                TanSolveResult.NoRetry -> throw apiError(
                     HttpStatusCode.TooManyRequests,
                     "Too many failed confirmation attempt",
                     TalerErrorCode.BANK_TAN_RATE_LIMITED
