@@ -21,17 +21,18 @@ package tech.libeufin.bank.db
 
 import tech.libeufin.bank.*
 import tech.libeufin.common.*
-import tech.libeufin.common.crypto.*
+import tech.libeufin.common.crypto.PwCrypto
+import tech.libeufin.common.db.*
 import java.time.Instant
 
 /** Data access logic for accounts */
 class AccountDAO(private val db: Database) {
     /** Result status of account creation */
-    enum class AccountCreationResult {
-        Success,
-        LoginReuse,
-        PayToReuse,
-        BonusBalanceInsufficient,
+    sealed interface AccountCreationResult {
+        data class Success(val payto: String): AccountCreationResult
+        data object LoginReuse: AccountCreationResult
+        data object PayToReuse: AccountCreationResult
+        data object BonusBalanceInsufficient: AccountCreationResult
     }
 
     /** Create new account */
@@ -49,9 +50,10 @@ class AccountDAO(private val db: Database) {
         bonus: TalerAmount,
         tanChannel: TanChannel?,
         // Whether to check [internalPaytoUri] for idempotency
-        checkPaytoIdempotent: Boolean
+        checkPaytoIdempotent: Boolean,
+        ctx: BankPaytoCtx
     ): AccountCreationResult = db.serializable { it ->
-        val now = Instant.now().toDbMicros() ?: throw faultyTimestampByBank()
+        val now = Instant.now().micros()
         it.transaction { conn ->
             val idempotent = conn.prepareStatement("""
                 SELECT password_hash, name=?
@@ -62,6 +64,7 @@ class AccountDAO(private val db: Database) {
                     AND is_public=?
                     AND is_taler_exchange=?
                     AND tan_channel IS NOT DISTINCT FROM ?::tan_enum
+                    ,internal_payto_uri, name
                 FROM customers 
                     JOIN bank_accounts
                         ON customer_id=owning_customer_id
@@ -79,13 +82,16 @@ class AccountDAO(private val db: Database) {
                 setString(9, tanChannel?.name)
                 setString(10, login)
                 oneOrNull { 
-                    PwCrypto.checkpw(password, it.getString(1)) && it.getBoolean(2)
+                    Pair(
+                        PwCrypto.checkpw(password, it.getString(1)) && it.getBoolean(2),
+                        it.getBankPayto("internal_payto_uri", "name", ctx)
+                    )
                 } 
             }
             
             if (idempotent != null) {
-                if (idempotent) {
-                    AccountCreationResult.Success
+                if (idempotent.first) {
+                    AccountCreationResult.Success(idempotent.second)
                 } else {
                     AccountCreationResult.LoginReuse
                 }
@@ -152,7 +158,7 @@ class AccountDAO(private val db: Database) {
                 if (bonus.value != 0L || bonus.frac != 0) {
                     conn.prepareStatement("""
                         SELECT out_balance_insufficient
-                        FROM bank_transaction(?,'admin','bonus',(?,?)::taler_amount,?,true)
+                        FROM bank_transaction(?,'admin','bonus',(?,?)::taler_amount,?,true,NULL)
                     """).run {
                         setString(1, internalPayto.canonical)
                         setLong(2, bonus.value)
@@ -165,12 +171,12 @@ class AccountDAO(private val db: Database) {
                                     conn.rollback()
                                     AccountCreationResult.BonusBalanceInsufficient
                                 }
-                                else -> AccountCreationResult.Success
+                                else -> AccountCreationResult.Success(internalPayto.bank(name, ctx))
                             }
                         }
                     }
                 } else {
-                    AccountCreationResult.Success
+                    AccountCreationResult.Success(internalPayto.bank(name, ctx))
                 }
             }
         }
@@ -189,15 +195,17 @@ class AccountDAO(private val db: Database) {
         login: String, 
         is2fa: Boolean
     ): AccountDeletionResult = db.serializable { conn ->
+        val now = Instant.now().micros()
         val stmt = conn.prepareStatement("""
             SELECT
               out_not_found,
               out_balance_not_zero,
               out_tan_required
-              FROM account_delete(?,?);
+              FROM account_delete(?,?,?)
         """)
         stmt.setString(1, login)
-        stmt.setBoolean(2, is2fa)
+        stmt.setLong(2, now)
+        stmt.setBoolean(3, is2fa)
         stmt.executeQuery().use {
             when {
                 !it.next() -> throw internalServerError("Deletion returned nothing.")
@@ -260,7 +268,7 @@ class AccountDAO(private val db: Database) {
             FROM customers
                 JOIN bank_accounts 
                 ON customer_id=owning_customer_id
-            WHERE login=?
+            WHERE login=? AND deleted_at IS NULL
         """).run {
             setString(1, login)
             oneOrNull {
@@ -383,7 +391,8 @@ class AccountDAO(private val db: Database) {
     ): AccountPatchAuthResult = db.serializable {
         it.transaction { conn ->
             val (currentPwh, tanRequired) = conn.prepareStatement("""
-                SELECT password_hash, (NOT ? AND tan_channel IS NOT NULL) FROM customers WHERE login=?
+                SELECT password_hash, (NOT ? AND tan_channel IS NOT NULL) 
+                FROM customers WHERE login=? AND deleted_at IS NULL
             """).run {
                 setBoolean(1, is2fa)
                 setString(2, login)
@@ -410,7 +419,7 @@ class AccountDAO(private val db: Database) {
     /** Get password hash of account [login] */
     suspend fun passwordHash(login: String): String? = db.conn { conn ->
         val stmt = conn.prepareStatement("""
-            SELECT password_hash FROM customers WHERE login=?
+            SELECT password_hash FROM customers WHERE login=? AND deleted_at IS NULL
         """)
         stmt.setString(1, login)
         stmt.oneOrNull { 
@@ -427,9 +436,8 @@ class AccountDAO(private val db: Database) {
              ,name
              ,is_taler_exchange
             FROM bank_accounts
-                JOIN customers 
-                ON customer_id=owning_customer_id
-                WHERE login=?
+                JOIN customers ON customer_id=owning_customer_id
+            WHERE login=?
         """)
         stmt.setString(1, login)
         stmt.oneOrNull {
@@ -458,6 +466,10 @@ class AccountDAO(private val db: Database) {
                 ,(max_debt).frac AS max_debt_frac
                 ,is_public
                 ,is_taler_exchange
+                ,CASE 
+                    WHEN deleted_at IS NOT NULL THEN 'deleted'
+                    ELSE 'active'
+                END as status
             FROM customers 
                 JOIN bank_accounts
                     ON customer_id=owning_customer_id
@@ -485,7 +497,8 @@ class AccountDAO(private val db: Database) {
                 ),
                 debit_threshold = it.getAmount("max_debt", db.bankCurrency),
                 is_public = it.getBoolean("is_public"),
-                is_taler_exchange = it.getBoolean("is_taler_exchange")
+                is_taler_exchange = it.getBoolean("is_taler_exchange"),
+                status = AccountStatus.valueOf(it.getString("status"))
             )
         }
     }
@@ -507,7 +520,7 @@ class AccountDAO(private val db: Database) {
               bank_account_id
               FROM bank_accounts JOIN customers
                 ON owning_customer_id = customer_id 
-              WHERE is_public=true AND name LIKE ? AND
+              WHERE is_public=true AND name LIKE ? AND deleted_at IS NULL AND
             """,
             {
                 setString(1, params.loginFilter)
@@ -548,6 +561,10 @@ class AccountDAO(private val db: Database) {
             ,is_taler_exchange
             ,internal_payto_uri
             ,bank_account_id
+            ,CASE 
+                WHEN deleted_at IS NOT NULL THEN 'deleted'
+                ELSE 'active'
+            END as status
             FROM bank_accounts JOIN customers
               ON owning_customer_id = customer_id 
             WHERE name LIKE ? AND
@@ -573,6 +590,7 @@ class AccountDAO(private val db: Database) {
                 is_public = it.getBoolean("is_public"),
                 is_taler_exchange = it.getBoolean("is_taler_exchange"),
                 payto_uri = it.getBankPayto("internal_payto_uri", "name", ctx),
+                status = AccountStatus.valueOf(it.getString("status"))
             )
         }
 }
\ No newline at end of file
