@@ -21,6 +21,7 @@ package tech.libeufin.bank.db
 
 import tech.libeufin.bank.*
 import tech.libeufin.common.*
+import tech.libeufin.common.db.*
 import java.time.Instant
 
 /** Data access logic for cashout operations */
@@ -66,7 +67,7 @@ class CashoutDAO(private val db: Database) {
         stmt.setLong(5, amountCredit.value)
         stmt.setInt(6, amountCredit.frac)
         stmt.setString(7, subject)
-        stmt.setLong(8, now.toDbMicros() ?: throw faultyTimestampByBank())
+        stmt.setLong(8, now.micros())
         stmt.setBoolean(9, is2fa)
         stmt.executeQuery().use {
             when {
@@ -117,7 +118,7 @@ class CashoutDAO(private val db: Database) {
                 creation_time = it.getTalerTimestamp("creation_time"),
                 confirmation_time = when (val timestamp = it.getLong("confirmation_date")) {
                     0L -> null
-                    else -> TalerProtocolTimestamp(timestamp.microsToJavaInstant() ?: throw faultyTimestampByBank())
+                    else -> TalerProtocolTimestamp(timestamp.asInstant())
                 },
                 tan_channel = it.getString("tan_channel")?.run { TanChannel.valueOf(this) },
                 tan_info = it.getString("tan_info"),
