@@ -22,6 +22,7 @@ import tech.libeufin.bank.DecimalNumber
 import tech.libeufin.bank.db.TransactionDAO.BankTransactionResult
 import tech.libeufin.bank.db.WithdrawalDAO.WithdrawalCreationResult
 import tech.libeufin.common.*
+import tech.libeufin.common.db.oneOrNull
 import java.time.Instant
 import java.util.*
 import kotlin.test.assertEquals
@@ -53,7 +54,8 @@ class AmountTest {
                 subject = "test",
                 amount = due,
                 timestamp = Instant.now(),
-                is2fa = false
+                is2fa = false,
+                requestUid = null
             )
             val txBool = when (txRes) {
                 BankTransactionResult.BalanceInsufficient -> false
