@@ -18,19 +18,9 @@
  */
 
 import io.ktor.client.request.*
-import io.ktor.client.statement.*
 import io.ktor.http.*
-import io.ktor.server.testing.*
-import kotlinx.serialization.json.JsonElement
 import org.junit.Test
-import tech.libeufin.bank.*
 import tech.libeufin.common.*
-import java.time.Duration
-import java.time.Instant
-import java.util.*
-import kotlin.test.assertEquals
-import kotlin.test.assertNotNull
-import kotlin.test.assertNull
 
 class CommonApiTest {
     @Test
