@@ -137,8 +137,7 @@ class CoreBankTokenApiTest {
     @Test
     fun delete() = bankSetup { _ -> 
         // TODO test restricted
-        val token = client.post("/accounts/merchant/token") {
-            pwAuth("merchant")
+        val token = client.postA("/accounts/merchant/token") {
             json { "scope" to "readonly" }
         }.assertOkJson<TokenSuccessResponse>().access_token
         // Check OK
@@ -151,9 +150,7 @@ class CoreBankTokenApiTest {
         }.assertUnauthorized()
 
         // Checking merchant can still be served by basic auth, after token deletion.
-        client.get("/accounts/merchant") {
-            pwAuth("merchant")
-        }.assertOk()
+        client.getA("/accounts/merchant").assertOk()
     }
 }
 
@@ -174,7 +171,9 @@ class CoreBankAccountsApiTest {
             // Check idempotency
             client.post("/accounts") {
                 json(req)
-            }.assertOk()
+            }.assertOkJson<RegisterAccountResponse> {
+                assertEquals(payto, it.internal_payto_uri)
+            }
             // Check idempotency with payto
             client.post("/accounts") {
                 json(req) {
@@ -208,8 +207,9 @@ class CoreBankAccountsApiTest {
         // Testing idempotency
         client.post("/accounts") {
             json(req)
-        }.assertOk()
-
+        }.assertOkJson<RegisterAccountResponse> {
+            assertEquals(payto.full("Jane"), it.internal_payto_uri)
+        }
         // Check admin only debit_threshold
         obj {
             "username" to "bat"
@@ -327,6 +327,34 @@ class CoreBankAccountsApiTest {
         client.getA("/accounts/cashout_keep").assertOkJson<AccountData> {
             assertEquals(full, it.cashout_payto_uri)
         }
+
+        // Check input restrictions
+        obj {
+            "username" to "username"
+            "password" to "password"
+            "name" to "Name"
+        }.let { req ->
+            client.post("/accounts") {
+                json(req) { "username" to "bad/username" }
+            }.assertBadRequest()
+            client.post("/accounts") {
+                json(req) { "username" to " spaces " }
+            }.assertBadRequest()
+            client.post("/accounts") {
+                json(req) {
+                    "contact_data" to obj {
+                        "phone" to " +456"
+                    }
+                }
+            }.assertBadRequest()
+            client.post("/accounts") {
+                json(req) {
+                    "contact_data" to obj {
+                        "phone" to " test@mail.com"
+                    }
+                }
+            }.assertBadRequest()
+        }
     }
 
     // Test account created with bonus
@@ -421,28 +449,86 @@ class CoreBankAccountsApiTest {
             .assertConflict(TalerErrorCode.BANK_ACCOUNT_BALANCE_NOT_ZERO)
         // Successful deletion
         tx("john", "KUDOS:1", "customer")
-        // TODO remove with gc
-        db.conn { conn ->
-            val id = conn.prepareStatement("SELECT bank_account_id FROM bank_accounts JOIN customers ON customer_id=owning_customer_id WHERE login = ?").run {
-                setString(1, "john")
-                oneOrNull {
-                    it.getLong(1)
-                }!!
-            }
-            conn.prepareStatement("DELETE FROM bank_account_transactions WHERE bank_account_id=?").run {
-                setLong(1, id)
-                execute()
-            }
-        }
         client.deleteA("/accounts/john")
             .assertChallenge()
             .assertNoContent()
         // Account no longer exists
+        client.deleteA("/accounts/john").assertUnauthorized()
         client.delete("/accounts/john") {
             pwAuth("admin")
         }.assertNotFound(TalerErrorCode.BANK_UNKNOWN_ACCOUNT)
     }
 
+
+
+    @Test
+    fun softDelete() = bankSetup { db -> 
+        // Create all kind of operations
+        val token = client.postA("/accounts/customer/token") {
+            json { "scope" to "readonly" }
+        }.assertOkJson<TokenSuccessResponse>().access_token
+        val tx_id = client.postA("/accounts/customer/transactions") {
+            json {
+                "payto_uri" to "$exchangePayto?message=payout"
+                "amount" to "KUDOS:0.3"
+            }
+        }.assertOkJson<TransactionCreateResponse>().row_id
+        val withdrawal_id = client.postA("/accounts/customer/withdrawals") {
+            json { "amount" to "KUDOS:9.0" } 
+        }.assertOkJson<BankAccountCreateWithdrawalResponse>().withdrawal_id
+        fillCashoutInfo("customer")
+        val cashout_id = client.postA("/accounts/customer/cashouts") {
+            json {
+                "request_uid" to ShortHashCode.rand()
+                "amount_debit" to "KUDOS:1"
+                "amount_credit" to convert("KUDOS:1")
+            }
+        }.assertOkJson<CashoutResponse>().cashout_id
+        fillTanInfo("customer")
+        val tan_id = client.postA("/accounts/customer/transactions") {
+            json {
+                "payto_uri" to "$exchangePayto?message=payout"
+                "amount" to "KUDOS:0.3"
+            }
+        }.assertAcceptedJson<TanChallenge>().challenge_id
+
+        // Delete account
+        tx("merchant", "KUDOS:1", "customer")
+        assertBalance("customer", "+KUDOS:0")
+        client.deleteA("/accounts/customer")
+            .assertChallenge()
+            .assertNoContent()
+        
+        // Check account can no longer login
+        client.delete("/accounts/customer/token") {
+            headers["Authorization"] = "Bearer secret-token:$token"
+        }.assertUnauthorized()
+        client.getA("/accounts/customer/transactions/$tx_id").assertUnauthorized()
+        client.getA("/accounts/customer/cashouts/$cashout_id").assertUnauthorized()
+        client.postA("/accounts/customer/withdrawals/$withdrawal_id/confirm").assertUnauthorized()
+
+        // But admin can still see existing operations
+        client.get("/accounts/customer/transactions/$tx_id") {
+            pwAuth("admin")
+        }.assertOkJson<BankAccountTransactionInfo>()
+        client.get("/accounts/customer/cashouts/$cashout_id") {
+            pwAuth("admin")
+        }.assertOkJson<CashoutStatusResponse>()
+        client.get("/withdrawals/$withdrawal_id")
+            .assertOkJson<WithdrawalPublicInfo>()
+
+        // GC
+        db.gc.collect(Instant.now(), Duration.ZERO, Duration.ZERO, Duration.ZERO)
+        client.get("/accounts/customer/transactions/$tx_id") {
+            pwAuth("admin")
+        }.assertNotFound(TalerErrorCode.BANK_TRANSACTION_NOT_FOUND)
+        client.get("/accounts/customer/cashouts/$cashout_id") {
+            pwAuth("admin")
+        }.assertNotFound(TalerErrorCode.BANK_TRANSACTION_NOT_FOUND)
+        client.get("/withdrawals/$withdrawal_id")
+            .assertNotFound(TalerErrorCode.BANK_TRANSACTION_NOT_FOUND)
+    }
+
     // Test admin-only account deletion
     @Test
     fun deleteRestricted() = bankSetup(conf = "test_restrict.conf") { _ -> 
@@ -510,7 +596,6 @@ class CoreBankAccountsApiTest {
             json(req)
         }.assertNoContent()
 
-        
         checkAdminOnly(
             obj(req) { "debit_threshold" to "KUDOS:100" },
             TalerErrorCode.BANK_NON_ADMIN_PATCH_DEBT_LIMIT
@@ -700,19 +785,32 @@ class CoreBankAccountsApiTest {
 
     // GET /public-accounts and GET /accounts
     @Test
-    fun list() = bankSetup(conf = "test_no_conversion.conf") { _ -> 
+    fun list() = bankSetup(conf = "test_no_conversion.conf") { db -> 
         authRoutine(HttpMethod.Get, "/accounts", requireAdmin = true)
         // Remove default accounts
-        listOf("merchant", "exchange", "customer").forEach {
+        val defaultAccounts = listOf("merchant", "exchange", "customer")
+        defaultAccounts.forEach {
             client.delete("/accounts/$it") {
                 pwAuth("admin")
             }.assertNoContent()
         }
+        client.get("/accounts") {
+            pwAuth("admin")
+        }.assertOkJson<ListBankAccountsResponse> {
+            for (account in it.accounts) {
+                if (defaultAccounts.contains(account.username)) {
+                    assertEquals(AccountStatus.deleted, account.status)
+                } else {
+                    assertEquals(AccountStatus.active, account.status)
+                }
+            }
+        }
+        db.gc.collect(Instant.now(), Duration.ZERO, Duration.ZERO, Duration.ZERO)
         // Check error when no public accounts
         client.get("/public-accounts").assertNoContent()
         client.get("/accounts") {
             pwAuth("admin")
-        }.assertOk()
+        }.assertOkJson<ListBankAccountsResponse>()
         
         // Gen some public and private accounts
         repeat(5) {
@@ -726,22 +824,18 @@ class CoreBankAccountsApiTest {
             }.assertOk()
         }
         // All public
-        client.get("/public-accounts").run {
-            assertOk()
-            val obj = json<PublicAccountsResponse>()
-            assertEquals(3, obj.public_accounts.size)
-            obj.public_accounts.forEach {
+        client.get("/public-accounts").assertOkJson<PublicAccountsResponse> {
+            assertEquals(3, it.public_accounts.size)
+            it.public_accounts.forEach {
                 assertEquals(0, it.username.toInt() % 2)
             }
         }
         // All accounts
         client.get("/accounts?delta=10"){
             pwAuth("admin")
-        }.run {
-            assertOk()
-            val obj = json<ListBankAccountsResponse>()
-            assertEquals(6, obj.accounts.size)
-            obj.accounts.forEachIndexed { idx, it ->
+        }.assertOkJson<ListBankAccountsResponse> {
+            assertEquals(6, it.accounts.size)
+            it.accounts.forEachIndexed { idx, it ->
                 if (idx == 0) {
                     assertEquals("admin", it.username)
                 } else {
@@ -752,11 +846,9 @@ class CoreBankAccountsApiTest {
         // Filtering
         client.get("/accounts?filter_name=3"){
             pwAuth("admin")
-        }.run {
-            assertOk()
-            val obj = json<ListBankAccountsResponse>()
-            assertEquals(1, obj.accounts.size)
-            assertEquals("3", obj.accounts[0].username)
+        }.assertOkJson<ListBankAccountsResponse> {
+            assertEquals(1, it.accounts.size)
+            assertEquals("3", it.accounts[0].username)
         }
     }
 
@@ -775,7 +867,7 @@ class CoreBankTransactionsApiTest {
     // GET /transactions
     @Test
     fun history() = bankSetup { _ -> 
-        authRoutine(HttpMethod.Get, "/accounts/merchant/transactions")
+        authRoutine(HttpMethod.Get, "/accounts/merchant/transactions", allowAdmin = true)
         historyRoutine<BankAccountTransactionsResponse>(
             url = "/accounts/customer/transactions",
             ids = { it.transactions.map { it.row_id } },
@@ -813,7 +905,7 @@ class CoreBankTransactionsApiTest {
     // GET /transactions/T_ID
     @Test
     fun testById() = bankSetup { _ -> 
-        authRoutine(HttpMethod.Get, "/accounts/merchant/transactions/42")
+        authRoutine(HttpMethod.Get, "/accounts/merchant/transactions/42", allowAdmin = true)
 
         // Create transaction
         tx("merchant", "KUDOS:0.3", "exchange", "tx")
@@ -851,6 +943,28 @@ class CoreBankTransactionsApiTest {
                 assertEquals(TalerAmount("KUDOS:0.3"), tx.amount)
             }
         }
+
+        // Check idempotency
+        ShortHashCode.rand().let { requestUid ->
+            val id = client.postA("/accounts/merchant/transactions") {
+                json(valid_req) {
+                    "request_uid" to requestUid
+                }
+            }.assertOkJson<TransactionCreateResponse>().row_id
+            client.postA("/accounts/merchant/transactions") {
+                json(valid_req) {
+                    "request_uid" to requestUid
+                }
+            }.assertOkJson<TransactionCreateResponse> {
+                assertEquals(id, it.row_id)
+            }
+            client.postA("/accounts/merchant/transactions") {
+                json(valid_req) {
+                    "request_uid" to requestUid
+                    "amount" to "KUDOS:42"
+                }
+            }.assertConflict(TalerErrorCode.BANK_TRANSFER_REQUEST_UID_REUSED)
+        }
         
         // Check amount in payto_uri
         client.postA("/accounts/merchant/transactions") {
@@ -975,6 +1089,31 @@ class CoreBankTransactionsApiTest {
             assertBalance("merchant", "+KUDOS:2")
             assertBalance("customer", "+KUDOS:1")
         }
+
+        // Check 2fa idempotency
+        val req = obj {
+            "payto_uri" to "$customerPayto?message=tan+check&amount=KUDOS:1"
+            "request_uid" to ShortHashCode.rand()
+        }
+        val id = client.postA("/accounts/merchant/transactions") {
+            json(req)
+        }.assertChallenge { _,_->
+            assertBalance("merchant", "+KUDOS:2")
+            assertBalance("customer", "+KUDOS:1")
+        }.assertOkJson <TransactionCreateResponse> { 
+            assertBalance("merchant", "+KUDOS:1")
+            assertBalance("customer", "+KUDOS:2")
+        }.row_id
+        client.postA("/accounts/merchant/transactions") {
+            json(req)
+        }.assertOkJson<TransactionCreateResponse> {
+            assertEquals(id, it.row_id)
+        }
+        client.postA("/accounts/merchant/transactions") {
+            json(req) {
+                "payto_uri" to "$customerPayto?message=tan+chec2k&amount=KUDOS:1"
+            }
+        }.assertConflict(TalerErrorCode.BANK_TRANSFER_REQUEST_UID_REUSED)
     }
 }
 
@@ -1010,9 +1149,7 @@ class CoreBankWithdrawalApiTest {
         client.postA("/accounts/merchant/withdrawals") {
             json { "amount" to amount}
         }.assertOkJson<BankAccountCreateWithdrawalResponse> {
-            client.get("/withdrawals/${it.withdrawal_id}") {
-                pwAuth("merchant")
-            }.assertOkJson<WithdrawalPublicInfo> {
+            client.get("/withdrawals/${it.withdrawal_id}").assertOkJson<WithdrawalPublicInfo> {
                 assertEquals(amount, it.amount)
             }
         }
@@ -1036,7 +1173,7 @@ class CoreBankWithdrawalApiTest {
         client.postA("/accounts/merchant/withdrawals") {
             json { "amount" to "KUDOS:1" } 
         }.assertOkJson<BankAccountCreateWithdrawalResponse> {
-            val uuid = it.taler_withdraw_uri.split("/").last()
+            val uuid = it.withdrawal_id
 
             // Check err
             client.postA("/accounts/merchant/withdrawals/$uuid/confirm")
@@ -1047,7 +1184,7 @@ class CoreBankWithdrawalApiTest {
         client.postA("/accounts/merchant/withdrawals") {
             json { "amount" to "KUDOS:1" } 
         }.assertOkJson<BankAccountCreateWithdrawalResponse> {
-            val uuid = it.taler_withdraw_uri.split("/").last()
+            val uuid = it.withdrawal_id
             withdrawalSelect(uuid)
 
             // Check OK
@@ -1060,7 +1197,7 @@ class CoreBankWithdrawalApiTest {
         client.postA("/accounts/merchant/withdrawals") {
             json { "amount" to "KUDOS:1" } 
         }.assertOkJson<BankAccountCreateWithdrawalResponse> {
-            val uuid = it.taler_withdraw_uri.split("/").last()
+            val uuid = it.withdrawal_id
             withdrawalSelect(uuid)
             client.postA("/accounts/merchant/withdrawals/$uuid/abort").assertNoContent()
 
@@ -1073,7 +1210,7 @@ class CoreBankWithdrawalApiTest {
         client.postA("/accounts/merchant/withdrawals") {
             json { "amount" to "KUDOS:5" } 
         }.assertOkJson<BankAccountCreateWithdrawalResponse> {
-            val uuid = it.taler_withdraw_uri.split("/").last()
+            val uuid = it.withdrawal_id
             withdrawalSelect(uuid)
 
             // Send too much money
@@ -1089,7 +1226,7 @@ class CoreBankWithdrawalApiTest {
         client.postA("/accounts/customer/withdrawals") {
             json { "amount" to "KUDOS:1" } 
         }.assertOkJson<BankAccountCreateWithdrawalResponse> {
-            val uuid = it.taler_withdraw_uri.split("/").last()
+            val uuid = it.withdrawal_id
             withdrawalSelect(uuid)
 
             // Check error
@@ -1111,7 +1248,7 @@ class CoreBankWithdrawalApiTest {
         client.postA("/accounts/merchant/withdrawals") {
             json { "amount" to "KUDOS:1" } 
         }.assertOkJson<BankAccountCreateWithdrawalResponse> {
-            val uuid = it.taler_withdraw_uri.split("/").last()
+            val uuid = it.withdrawal_id
             withdrawalSelect(uuid)
 
             client.postA("/accounts/merchant/withdrawals/$uuid/confirm")
@@ -1122,7 +1259,6 @@ class CoreBankWithdrawalApiTest {
     }
 }
 
-
 class CoreBankCashoutApiTest {
     // POST /accounts/{USERNAME}/cashouts
     @Test
@@ -1143,9 +1279,16 @@ class CoreBankCashoutApiTest {
         fillCashoutInfo("customer")
 
         // Check OK
+        val id = client.postA("/accounts/customer/cashouts") {
+            json(req) 
+        }.assertOkJson<CashoutResponse>().cashout_id
+
+        // Check idempotent
         client.postA("/accounts/customer/cashouts") {
             json(req) 
-        }.assertOkJson<CashoutResponse>()
+        }.assertOkJson<CashoutResponse> {
+            assertEquals(id, it.cashout_id)
+        }
 
         // Trigger conflict due to reused request_uid
         client.postA("/accounts/customer/cashouts") {
@@ -1205,7 +1348,7 @@ class CoreBankCashoutApiTest {
     // GET /accounts/{USERNAME}/cashouts/{CASHOUT_ID}
     @Test
     fun get() = bankSetup { _ ->
-        authRoutine(HttpMethod.Get, "/accounts/merchant/cashouts/42")
+        authRoutine(HttpMethod.Get, "/accounts/merchant/cashouts/42", allowAdmin = true)
         fillCashoutInfo("customer")
 
         val amountDebit = TalerAmount("KUDOS:1.5")
@@ -1253,7 +1396,7 @@ class CoreBankCashoutApiTest {
     // GET /accounts/{USERNAME}/cashouts
     @Test
     fun history() = bankSetup { _ ->
-        authRoutine(HttpMethod.Get, "/accounts/merchant/cashouts")
+        authRoutine(HttpMethod.Get, "/accounts/merchant/cashouts", allowAdmin = true)
         historyRoutine<Cashouts>(
             url = "/accounts/customer/cashouts",
             ids = { it.cashouts.map { it.cashout_id } },
