@@ -19,8 +19,6 @@
 
 import io.ktor.http.*
 import org.junit.Test
-import tech.libeufin.bank.IncomingHistory
-import tech.libeufin.bank.OutgoingHistory
 import tech.libeufin.common.*
 
 class WireGatewayApiTest {
