@@ -27,12 +27,15 @@ import tech.libeufin.bank.*
 import tech.libeufin.bank.db.AccountDAO.AccountCreationResult
 import tech.libeufin.bank.db.Database
 import tech.libeufin.common.*
+import tech.libeufin.common.db.dbInit
+import tech.libeufin.common.db.pgDataSource
 import java.nio.file.NoSuchFileException
 import kotlin.io.path.Path
 import kotlin.io.path.deleteExisting
 import kotlin.io.path.readText
 import kotlin.random.Random
 import kotlin.test.assertEquals
+import kotlin.test.assertIs
 import kotlin.test.assertNotNull
 
 /* ----- Setup ----- */
@@ -56,109 +59,107 @@ fun genTmpPayTo(): IbanPayto {
 fun setup(
     conf: String = "test.conf",
     lambda: suspend (Database, BankConfig) -> Unit
-) {
+) = runBlocking {
     val config = talerConfig(Path("conf/$conf"))
     val dbCfg = config.loadDbConfig()
     val ctx = config.loadBankConfig()
-    Database(dbCfg.dbConnStr, ctx.regionalCurrency, ctx.fiatCurrency).use {
-        runBlocking {
-            it.conn { conn ->
-                resetDatabaseTables(conn, dbCfg, "libeufin-nexus")
-                initializeDatabaseTables(conn, dbCfg, "libeufin-nexus")
-                resetDatabaseTables(conn, dbCfg, "libeufin-bank")
-                initializeDatabaseTables(conn, dbCfg, "libeufin-bank")
-                val sqlProcedures = Path("${dbCfg.sqlDir}/libeufin-conversion-setup.sql")
-                conn.execSQLUpdate(sqlProcedures.readText())
-            }
-            lambda(it, ctx)
+    pgDataSource(dbCfg.dbConnStr).run {
+        dbInit(dbCfg, "libeufin-nexus", true)
+        dbInit(dbCfg, "libeufin-bank", true)
+    }
+    Database(dbCfg, ctx.regionalCurrency, ctx.fiatCurrency).use {
+        it.conn { conn ->
+            val sqlProcedures = Path("${dbCfg.sqlDir}/libeufin-conversion-setup.sql")
+            conn.execSQLUpdate(sqlProcedures.readText())
         }
+        lambda(it, ctx)
     }
 }
 
 fun bankSetup(
     conf: String = "test.conf",    
     lambda: suspend ApplicationTestBuilder.(Database) -> Unit
-) {
-    setup(conf) { db, ctx -> 
-        // Creating the exchange and merchant accounts first.
-        val bonus = TalerAmount("KUDOS:0")
-        assertEquals(AccountCreationResult.Success, db.account.create(
-            login = "merchant",
-            password = "merchant-password",
-            name = "Merchant",
-            internalPayto = merchantPayto,
-            maxDebt = TalerAmount("KUDOS:10"),
-            isTalerExchange = false,
-            isPublic = false,
-            bonus = bonus,
-            checkPaytoIdempotent = false,
-            email = null,
-            phone = null,
-            cashoutPayto = null,
-            tanChannel = null   
-        ))
-        assertEquals(AccountCreationResult.Success, db.account.create(
-            login = "exchange",
-            password = "exchange-password",
-            name = "Exchange",
-            internalPayto = exchangePayto,
-            maxDebt = TalerAmount("KUDOS:10"),
-            isTalerExchange = true,
-            isPublic = false,
-            bonus = bonus,
-            checkPaytoIdempotent = false,
-            email = null,
-            phone = null,
-            cashoutPayto = null,
-            tanChannel = null   
-        ))
-        assertEquals(AccountCreationResult.Success, db.account.create(
-            login = "customer",
-            password = "customer-password",
-            name = "Customer",
-            internalPayto = customerPayto,
-            maxDebt = TalerAmount("KUDOS:10"),
-            isTalerExchange = false,
-            isPublic = false,
-            bonus = bonus,
-            checkPaytoIdempotent = false,
-            email = null,
-            phone = null,
-            cashoutPayto = null,
-            tanChannel = null   
-        ))
-        // Create admin account
-        assertEquals(AccountCreationResult.Success, createAdminAccount(db, ctx, "admin-password"))
-        testApplication {
-            application {
-                corebankWebApp(db, ctx)
-            }
-            if (ctx.allowConversion) {
-                // Set conversion rates
-                client.post("/conversion-info/conversion-rate") {
-                    pwAuth("admin")
-                    json {
-                        "cashin_ratio" to "0.8"
-                        "cashin_fee" to "KUDOS:0.02"
-                        "cashin_tiny_amount" to "KUDOS:0.01"
-                        "cashin_rounding_mode" to "nearest"
-                        "cashin_min_amount" to "EUR:0"
-                        "cashout_ratio" to "1.25"
-                        "cashout_fee" to "EUR:0.003"
-                        "cashout_tiny_amount" to "EUR:0.00000001"
-                        "cashout_rounding_mode" to "zero"
-                        "cashout_min_amount" to "KUDOS:0.1"
-                    }
-                }.assertNoContent()
-            }
-            lambda(db)
+) = setup(conf) { db, cfg -> 
+    // Creating the exchange and merchant accounts first.
+    val bonus = TalerAmount("KUDOS:0")
+    assertIs<AccountCreationResult.Success>(db.account.create(
+        login = "merchant",
+        password = "merchant-password",
+        name = "Merchant",
+        internalPayto = merchantPayto,
+        maxDebt = TalerAmount("KUDOS:10"),
+        isTalerExchange = false,
+        isPublic = false,
+        bonus = bonus,
+        checkPaytoIdempotent = false,
+        email = null,
+        phone = null,
+        cashoutPayto = null,
+        tanChannel = null,
+        ctx = cfg.payto  
+    ))
+    assertIs<AccountCreationResult.Success>(db.account.create(
+        login = "exchange",
+        password = "exchange-password",
+        name = "Exchange",
+        internalPayto = exchangePayto,
+        maxDebt = TalerAmount("KUDOS:10"),
+        isTalerExchange = true,
+        isPublic = false,
+        bonus = bonus,
+        checkPaytoIdempotent = false,
+        email = null,
+        phone = null,
+        cashoutPayto = null,
+        tanChannel = null,
+        ctx = cfg.payto   
+    ))
+    assertIs<AccountCreationResult.Success>(db.account.create(
+        login = "customer",
+        password = "customer-password",
+        name = "Customer",
+        internalPayto = customerPayto,
+        maxDebt = TalerAmount("KUDOS:10"),
+        isTalerExchange = false,
+        isPublic = false,
+        bonus = bonus,
+        checkPaytoIdempotent = false,
+        email = null,
+        phone = null,
+        cashoutPayto = null,
+        tanChannel = null,
+        ctx = cfg.payto
+    ))
+    // Create admin account
+    assertIs<AccountCreationResult.Success>(createAdminAccount(db, cfg, "admin-password"))
+    testApplication {
+        application {
+            corebankWebApp(db, cfg)
         }
+        if (cfg.allowConversion) {
+            // Set conversion rates
+            client.post("/conversion-info/conversion-rate") {
+                pwAuth("admin")
+                json {
+                    "cashin_ratio" to "0.8"
+                    "cashin_fee" to "KUDOS:0.02"
+                    "cashin_tiny_amount" to "KUDOS:0.01"
+                    "cashin_rounding_mode" to "nearest"
+                    "cashin_min_amount" to "EUR:0"
+                    "cashout_ratio" to "1.25"
+                    "cashout_fee" to "EUR:0.003"
+                    "cashout_tiny_amount" to "EUR:0.00000001"
+                    "cashout_rounding_mode" to "zero"
+                    "cashout_min_amount" to "KUDOS:0.1"
+                }
+            }.assertNoContent()
+        }
+        lambda(db)
     }
 }
 
-fun dbSetup(lambda: suspend (Database) -> Unit) {
+fun dbSetup(lambda: suspend (Database) -> Unit) =
     setup { db, _ -> lambda(db) }
-}
 
 /* ----- Common actions ----- */
 
@@ -315,33 +316,6 @@ suspend fun tanCode(info: String): String? {
 
 /* ----- Assert ----- */
 
-suspend fun HttpResponse.assertStatus(status: HttpStatusCode, err: TalerErrorCode?): HttpResponse {
-    assertEquals(status, this.status, "$err")
-    if (err != null) {
-        val body = json<TalerError>()
-        assertEquals(err.code, body.code)
-    }
-    return this
-}
-suspend fun HttpResponse.assertOk(): HttpResponse
-    = assertStatus(HttpStatusCode.OK, null)
-suspend fun HttpResponse.assertNoContent(): HttpResponse 
-    = assertStatus(HttpStatusCode.NoContent, null)
-suspend fun HttpResponse.assertAccepted(): HttpResponse 
-    = assertStatus(HttpStatusCode.Accepted, null)
-suspend fun HttpResponse.assertNotFound(err: TalerErrorCode): HttpResponse 
-    = assertStatus(HttpStatusCode.NotFound, err)
-suspend fun HttpResponse.assertUnauthorized(err: TalerErrorCode = TalerErrorCode.GENERIC_UNAUTHORIZED): HttpResponse 
-    = assertStatus(HttpStatusCode.Unauthorized, err)
-suspend fun HttpResponse.assertConflict(err: TalerErrorCode): HttpResponse 
-    = assertStatus(HttpStatusCode.Conflict, err)
-suspend fun HttpResponse.assertBadRequest(err: TalerErrorCode = TalerErrorCode.GENERIC_JSON_INVALID): HttpResponse 
-    = assertStatus(HttpStatusCode.BadRequest, err)
-suspend fun HttpResponse.assertForbidden(err: TalerErrorCode): HttpResponse 
-    = assertStatus(HttpStatusCode.Forbidden, err)
-suspend fun HttpResponse.assertNotImplemented(err: TalerErrorCode = TalerErrorCode.END): HttpResponse 
-    = assertStatus(HttpStatusCode.NotImplemented, err)
-
 suspend fun HttpResponse.maybeChallenge(): HttpResponse {
     return if (this.status == HttpStatusCode.Accepted) {
         this.assertChallenge()
@@ -477,12 +451,6 @@ fun HttpRequestBuilder.pwAuth(username: String? = null) {
 
 /* ----- Random data generation ----- */
 
-fun randBase32Crockford(length: Int) = Base32Crockford.encode(randBytes(length))
-
-fun randIncomingSubject(reservePub: EddsaPublicKey): String {
-    return "$reservePub"
-}
-
-fun randOutgoingSubject(wtid: ShortHashCode, url: ExchangeUrl): String {
-    return "$wtid $url"
-}
\ No newline at end of file
+fun randBase32Crockford(length: Int) = Base32Crockford.encode(ByteArray(length).rand())
+fun randIncomingSubject(reservePub: EddsaPublicKey): String = "$reservePub"
+fun randOutgoingSubject(wtid: ShortHashCode, url: ExchangeUrl): String = "$wtid $url"
\ No newline at end of file
