@@ -20,4 +20,10 @@ package tech.libeufin.common
 
 // DB
 const val MIN_VERSION: Int = 14
-const val SERIALIZATION_RETRY: Int = 10
\ No newline at end of file
+const val SERIALIZATION_RETRY: Int = 10
+
+// Security
+const val MAX_BODY_LENGTH: Long = 4 * 1024 // 4kB
+
+// API version
+const val WIRE_GATEWAY_API_VERSION: String = "0:2:0"
\ No newline at end of file
