@@ -142,6 +142,7 @@ COMMENT ON FUNCTION account_balance_is_sufficient IS 'Check if an account have e
 
 CREATE FUNCTION account_delete(
   IN in_login TEXT,
+  IN in_now INT8,
   IN in_is_tan BOOLEAN,
   OUT out_not_found BOOLEAN,
   OUT out_balance_not_zero BOOLEAN,
@@ -150,45 +151,26 @@ CREATE FUNCTION account_delete(
 LANGUAGE plpgsql AS $$
 DECLARE
 my_customer_id INT8;
-my_balance_val INT8;
-my_balance_frac INT4;
 BEGIN
--- check if login exists and if 2FA is required
-SELECT customer_id, (NOT in_is_tan AND tan_channel IS NOT NULL) 
-  INTO my_customer_id, out_tan_required
-  FROM customers
-  WHERE login = in_login;
-IF NOT FOUND THEN
-  out_not_found=TRUE;
-  RETURN;
-END IF;
-
--- get the balance
-SELECT
-  (balance).val as balance_val,
-  (balance).frac as balance_frac
-  INTO
-    my_balance_val,
-    my_balance_frac
-  FROM bank_accounts
-  WHERE owning_customer_id = my_customer_id;
-IF NOT FOUND THEN
-  RAISE EXCEPTION 'Invariant failed: customer lacks bank account';
-END IF;
-
--- check that balance is zero.
-IF my_balance_val != 0 OR my_balance_frac != 0 THEN
-  out_balance_not_zero=TRUE;
-  RETURN;
-END IF;
-
--- check tan required
-IF out_tan_required THEN
+-- check if account exists, has zero balance and if 2FA is required
+SELECT 
+   customer_id
+  ,(NOT in_is_tan AND tan_channel IS NOT NULL)
+  ,((balance).val != 0 OR (balance).frac != 0)
+  INTO 
+     my_customer_id
+    ,out_tan_required
+    ,out_balance_not_zero
+  FROM customers 
+    JOIN bank_accounts ON owning_customer_id = customer_id
+  WHERE login = in_login AND deleted_at IS NULL;
+IF NOT FOUND OR out_balance_not_zero OR out_tan_required THEN
+  out_not_found=NOT FOUND;
   RETURN;
 END IF;
 
 -- actual deletion
-DELETE FROM customers WHERE login = in_login;
+UPDATE customers SET deleted_at = in_now WHERE customer_id = my_customer_id;
 END $$;
 COMMENT ON FUNCTION account_delete IS 'Deletes an account if the balance is zero';
 
@@ -310,11 +292,9 @@ SELECT
   FROM bank_accounts 
       JOIN customers 
         ON customer_id=owning_customer_id
-  WHERE login = in_username;
-IF NOT FOUND THEN
-  out_debtor_not_found=TRUE;
-  RETURN;
-ELSIF out_debtor_not_exchange THEN
+  WHERE login = in_username AND deleted_at IS NULL;
+IF NOT FOUND OR out_debtor_not_exchange THEN
+  out_debtor_not_found=NOT FOUND;
   RETURN;
 END IF;
 -- Find receiver bank account id
@@ -323,10 +303,8 @@ SELECT
   INTO receiver_bank_account_id, out_both_exchanges
   FROM bank_accounts
   WHERE internal_payto_uri = in_credit_account_payto;
-IF NOT FOUND THEN
-  out_creditor_not_found=TRUE;
-  RETURN;
-ELSIF out_both_exchanges THEN
+IF NOT FOUND OR out_both_exchanges THEN
+  out_creditor_not_found=NOT FOUND;
   RETURN;
 END IF;
 -- Perform bank transfer
@@ -392,11 +370,9 @@ SELECT
   FROM bank_accounts 
       JOIN customers 
         ON customer_id=owning_customer_id
-  WHERE login = in_username;
-IF NOT FOUND THEN
-  out_creditor_not_found=TRUE;
-  RETURN;
-ELSIF out_creditor_not_exchange THEN
+  WHERE login = in_username AND deleted_at IS NULL;
+IF NOT FOUND OR out_creditor_not_exchange THEN
+  out_creditor_not_found=NOT FOUND;
   RETURN;
 END IF;
 -- Find sender bank account id
@@ -405,10 +381,8 @@ SELECT
   INTO sender_bank_account_id, out_both_exchanges
   FROM bank_accounts
   WHERE internal_payto_uri = in_debit_account_payto;
-IF NOT FOUND THEN
-  out_debtor_not_found=TRUE;
-  RETURN;
-ELSIF out_both_exchanges THEN
+IF NOT FOUND OR out_both_exchanges THEN
+  out_debtor_not_found=NOT FOUND;
   RETURN;
 END IF;
 -- Perform bank transfer
@@ -443,6 +417,7 @@ CREATE FUNCTION bank_transaction(
   IN in_amount taler_amount,
   IN in_timestamp INT8,
   IN in_is_tan BOOLEAN,
+  IN in_request_uid BYTEA,
   -- Error status
   OUT out_creditor_not_found BOOLEAN,
   OUT out_debtor_not_found BOOLEAN,
@@ -450,13 +425,15 @@ CREATE FUNCTION bank_transaction(
   OUT out_balance_insufficient BOOLEAN,
   OUT out_creditor_admin BOOLEAN,
   OUT out_tan_required BOOLEAN,
+  OUT out_request_uid_reuse BOOLEAN,
   -- Success return
   OUT out_credit_bank_account_id INT8,
   OUT out_debit_bank_account_id INT8,
   OUT out_credit_row_id INT8,
   OUT out_debit_row_id INT8,
   OUT out_creditor_is_exchange BOOLEAN,
-  OUT out_debtor_is_exchange BOOLEAN
+  OUT out_debtor_is_exchange BOOLEAN,
+  OUT out_idempotent BOOLEAN
 )
 LANGUAGE plpgsql AS $$
 BEGIN
@@ -465,25 +442,38 @@ SELECT bank_account_id, is_taler_exchange, login='admin'
   INTO out_credit_bank_account_id, out_creditor_is_exchange, out_creditor_admin
   FROM bank_accounts
     JOIN customers ON customer_id=owning_customer_id
-  WHERE internal_payto_uri = in_credit_account_payto;
-IF NOT FOUND THEN
-  out_creditor_not_found=TRUE;
-  RETURN;
-ELSIF out_creditor_admin THEN
+  WHERE internal_payto_uri = in_credit_account_payto AND deleted_at IS NULL;
+IF NOT FOUND OR out_creditor_admin THEN
+  out_creditor_not_found=NOT FOUND;
   RETURN;
 END IF;
--- Find debit bank account id and check it's a different account
+-- Find debit bank account ID and check it's a different account and if 2FA is required
 SELECT bank_account_id, is_taler_exchange, out_credit_bank_account_id=bank_account_id, NOT in_is_tan AND tan_channel IS NOT NULL
   INTO out_debit_bank_account_id, out_debtor_is_exchange, out_same_account, out_tan_required
   FROM bank_accounts 
     JOIN customers ON customer_id=owning_customer_id
-  WHERE login = in_debit_account_username;
-IF NOT FOUND THEN
-  out_debtor_not_found=TRUE;
-  RETURN;
-ELSIF out_same_account OR out_tan_required THEN
+  WHERE login = in_debit_account_username AND deleted_at IS NULL;
+IF NOT FOUND OR out_same_account THEN
+  out_debtor_not_found=NOT FOUND;
   RETURN;
 END IF;
+-- Check for idempotence and conflict
+IF in_request_uid IS NOT NULL THEN
+  SELECT (amount != in_amount
+      OR subject != in_subject 
+      OR bank_account_id != out_debit_bank_account_id), bank_transaction
+    INTO out_request_uid_reuse, out_debit_row_id
+    FROM bank_transaction_operations
+      JOIN bank_account_transactions ON bank_transaction = bank_transaction_id
+    WHERE request_uid = in_request_uid;
+  IF found OR out_tan_required THEN
+    out_idempotent = found AND NOT out_request_uid_reuse;
+    RETURN;
+  END IF;
+ELSIF out_tan_required THEN
+  RETURN;
+END IF;
+
 -- Perform bank transfer
 SELECT
   transfer.out_balance_insufficient,
@@ -503,6 +493,11 @@ SELECT
     NULL,
     NULL
   ) as transfer;
+-- Store operation
+IF in_request_uid IS NOT NULL THEN
+  INSERT INTO bank_transaction_operations (request_uid, bank_transaction)  
+    VALUES (in_request_uid, out_debit_row_id);
+END IF;
 END $$;
 COMMENT ON FUNCTION bank_transaction IS 'Create a bank transaction';
 
@@ -525,11 +520,9 @@ SELECT bank_account_id, is_taler_exchange
   INTO account_id, out_account_is_exchange
   FROM bank_accounts
   JOIN customers ON bank_accounts.owning_customer_id = customers.customer_id
-  WHERE login=in_account_username;
-IF NOT FOUND THEN
-  out_account_not_found=TRUE;
-  RETURN;
-ELSIF out_account_is_exchange THEN
+  WHERE login=in_account_username AND deleted_at IS NULL;
+IF NOT FOUND OR out_account_is_exchange THEN
+  out_account_not_found=NOT FOUND;
   RETURN;
 END IF;
 
@@ -577,10 +570,8 @@ SELECT
   INTO not_selected, out_status, out_already_selected
   FROM taler_withdrawal_operations
   WHERE withdrawal_uuid=in_withdrawal_uuid;
-IF NOT FOUND THEN
-  out_no_op=TRUE;
-  RETURN;
-ELSIF out_already_selected THEN
+IF NOT FOUND OR out_already_selected THEN
+  out_no_op=NOT FOUND;
   RETURN;
 END IF;
 
@@ -598,10 +589,8 @@ IF not_selected THEN
     INTO out_account_is_not_exchange
     FROM bank_accounts
     WHERE internal_payto_uri=in_selected_exchange_payto;
-  IF NOT FOUND THEN
-    out_account_not_found=TRUE;
-    RETURN;
-  ELSIF out_account_is_not_exchange THEN
+  IF NOT FOUND OR out_account_is_not_exchange THEN
+    out_account_not_found=NOT FOUND;
     RETURN;
   END IF;
 
@@ -628,10 +617,8 @@ UPDATE taler_withdrawal_operations
   WHERE withdrawal_uuid=in_withdrawal_uuid
   RETURNING confirmation_done
   INTO out_already_confirmed;
-IF NOT FOUND THEN
-  out_no_op=TRUE;
-  RETURN;
-ELSIF out_already_confirmed THEN
+IF NOT FOUND OR out_already_confirmed THEN
+  out_no_op=NOT FOUND;
   RETURN;
 END IF;
 
@@ -684,27 +671,20 @@ SELECT
   FROM taler_withdrawal_operations
     JOIN bank_accounts ON wallet_bank_account=bank_account_id
     JOIN customers ON owning_customer_id=customer_id
-  WHERE withdrawal_uuid=in_withdrawal_uuid AND login=in_login;
-IF NOT FOUND THEN
-  out_no_op=TRUE;
-  RETURN;
-ELSIF already_confirmed OR out_aborted OR out_not_selected THEN
+  WHERE withdrawal_uuid=in_withdrawal_uuid AND login=in_login AND deleted_at IS NULL;
+IF NOT FOUND OR already_confirmed OR out_aborted OR out_not_selected THEN
+  out_no_op=NOT FOUND;
   RETURN;
 END IF;
 
--- sending the funds to the exchange, but needs first its bank account row ID
+-- Check exchange account then 2faa
 SELECT
   bank_account_id
   INTO exchange_bank_account_id
   FROM bank_accounts
   WHERE internal_payto_uri = selected_exchange_payto_local;
-IF NOT FOUND THEN
-  out_exchange_not_found=TRUE;
-  RETURN;
-END IF;
-
--- Check 2FA
-IF out_tan_required THEN
+IF NOT FOUND OR out_tan_required THEN
+  out_exchange_not_found=NOT FOUND;
   RETURN;
 END IF;
 
@@ -1162,7 +1142,7 @@ DECLARE
 account_id INT8;
 BEGIN
 -- Retrieve account id
-SELECT customer_id INTO account_id FROM customers WHERE login = in_login;
+SELECT customer_id INTO account_id FROM customers WHERE login = in_login AND deleted_at IS NULL;
 -- Create challenge
 INSERT INTO tan_challenges (
   body,
@@ -1214,7 +1194,7 @@ SELECT customer_id, tan_channel, CASE tan_channel
     WHEN 'email' THEN email
   END
 INTO account_id, out_tan_channel, out_tan_info
-FROM customers WHERE login = in_login;
+FROM customers WHERE login = in_login AND deleted_at IS NULL;
 
 -- Recover expiration date
 SELECT 
@@ -1274,7 +1254,7 @@ DECLARE
 account_id INT8;
 BEGIN
 -- Retrieve account id
-SELECT customer_id INTO account_id FROM customers WHERE login = in_login;
+SELECT customer_id INTO account_id FROM customers WHERE login = in_login AND deleted_at IS NULL;
 -- Check challenge
 UPDATE tan_challenges SET 
   confirmation_date = CASE 
@@ -1288,10 +1268,8 @@ RETURNING
   retry_counter <= 0 AND confirmation_date IS NULL,
   in_now_date >= expiration_date AND confirmation_date IS NULL
 INTO out_ok, out_no_retry, out_expired;
-IF NOT FOUND THEN
-  out_no_op = true;
-  RETURN;
-ELSIF NOT out_ok OR out_no_retry OR out_expired THEN
+IF NOT FOUND OR NOT out_ok OR out_no_retry OR out_expired THEN
+  out_no_op = NOT FOUND;
   RETURN;
 END IF;
 
@@ -1303,9 +1281,8 @@ END $$;
 COMMENT ON FUNCTION tan_challenge_try IS 'Try to confirm a challenge, return true if the challenge have been confirmed';
 
 CREATE FUNCTION stats_get_frame(
-  IN now TIMESTAMP,
+  IN date TIMESTAMP,
   IN in_timeframe stat_timeframe_enum,
-  IN which INTEGER,
   OUT cashin_count INT8,
   OUT cashin_regional_volume taler_amount,
   OUT cashin_fiat_volume taler_amount,
@@ -1318,19 +1295,8 @@ CREATE FUNCTION stats_get_frame(
   OUT taler_out_volume taler_amount
 )
 LANGUAGE plpgsql AS $$
-DECLARE
-  local_start_time TIMESTAMP;
 BEGIN
-  IF now IS NULL THEN
-    now = timezone('utc', now())::TIMESTAMP;
-  END IF;
-  local_start_time = CASE 
-    WHEN which IS NULL          THEN date_trunc(in_timeframe::text, now)
-    WHEN in_timeframe = 'hour'  THEN date_trunc('day'  , now) + make_interval(hours  => which)
-    WHEN in_timeframe = 'day'   THEN date_trunc('month', now) + make_interval(days   => which-1)
-    WHEN in_timeframe = 'month' THEN date_trunc('year' , now) + make_interval(months => which-1)
-    WHEN in_timeframe = 'year'  THEN make_date(which, 1, 1)::TIMESTAMP
-  END;
+  date = date_trunc(in_timeframe::text, date);
   SELECT 
     s.cashin_count
     ,(s.cashin_regional_volume).val
@@ -1367,7 +1333,7 @@ BEGIN
     ,taler_out_volume.frac
   FROM bank_stats AS s
   WHERE s.timeframe = in_timeframe 
-    AND s.start_time = local_start_time;
+    AND s.start_time = date;
 END $$;
 
 CREATE PROCEDURE stats_register_payment(
