@@ -23,10 +23,11 @@ import com.github.ajalt.clikt.core.CliktCommand
 import com.github.ajalt.clikt.parameters.groups.*
 import com.github.ajalt.clikt.parameters.options.*
 import io.ktor.client.*
+import io.ktor.client.plugins.*
 import tech.libeufin.common.*
 import tech.libeufin.common.crypto.*
-import tech.libeufin.ebics.*
 import tech.libeufin.nexus.ebics.*
+import tech.libeufin.nexus.ebics.EbicsKeyMng.Order.*
 import java.nio.file.*
 import java.time.Instant
 import kotlin.io.path.*
@@ -50,16 +51,6 @@ private fun loadOrGenerateClientKeys(path: Path): ClientPrivateKeysFile {
     return newKeys
 }
 
-/**
- * Expresses the type of keying message that the user wants
- * to send to the bank.
- */
-enum class KeysOrderType {
-    INI,
-    HIA,
-    HPB
-}
-
 /**
  * @return the "this" string with a space every two characters.
  */
@@ -81,8 +72,8 @@ fun String.spaceEachTwo() =
  * @return true if the user accepted, false otherwise.
  */
 private fun askUserToAcceptKeys(bankKeys: BankPublicKeysFile): Boolean {
-    val encHash = CryptoUtil.getEbicsPublicKeyHash(bankKeys.bank_encryption_public_key).toHexString()
-    val authHash = CryptoUtil.getEbicsPublicKeyHash(bankKeys.bank_authentication_public_key).toHexString()
+    val encHash = CryptoUtil.getEbicsPublicKeyHash(bankKeys.bank_encryption_public_key).encodeUpHex()
+    val authHash = CryptoUtil.getEbicsPublicKeyHash(bankKeys.bank_authentication_public_key).encodeUpHex()
     println("The bank has the following keys:")
     println("Encryption key: ${encHash.spaceEachTwo()}")
     println("Authentication key: ${authHash.spaceEachTwo()}")
@@ -91,43 +82,6 @@ private fun askUserToAcceptKeys(bankKeys: BankPublicKeysFile): Boolean {
     return userResponse == "yes, accept"
 }
 
-/**
- * Parses the HPB response and stores the bank keys as "NOT accepted" to disk.
- *
- * @param cfg used to get the location of the bank keys file.
- * @param bankKeys bank response to the HPB message.
- */
-private fun handleHpbResponse(
-    cfg: EbicsSetupConfig,
-    bankKeys: EbicsKeyManagementResponseContent
-) {
-    val hpbBytes = bankKeys.orderData // silences compiler.
-    if (hpbBytes == null) {
-        throw Exception("HPB content not found in a EBICS response with successful return codes.")
-    }
-    val hpbObj = try {
-        parseEbicsHpbOrder(hpbBytes.inputStream())
-    } catch (e: Exception) {
-        throw Exception("HPB response content seems invalid", e)
-    }
-    val encPub = try {
-        CryptoUtil.loadRsaPublicKey(hpbObj.encryptionPubKey.encoded)
-    } catch (e: Exception) {
-        throw Exception("Could not import bank encryption key from HPB response", e)
-    }
-    val authPub = try {
-        CryptoUtil.loadRsaPublicKey(hpbObj.authenticationPubKey.encoded)
-    } catch (e: Exception) {
-        throw Exception("Could not import bank authentication key from HPB response", e)
-    }
-    val json = BankPublicKeysFile(
-        bank_authentication_public_key = authPub,
-        bank_encryption_public_key = encPub,
-        accepted = false
-    )
-    persistBankKeys(json, cfg.bankPublicKeysFilename)
-}
-
 /**
  * Collects all the steps from generating the message, to
  * sending it to the bank, and finally updating the state
@@ -141,43 +95,58 @@ private fun handleHpbResponse(
  *        the --auto-accept-key CLI flag.
  */
 suspend fun doKeysRequestAndUpdateState(
-    cfg: EbicsSetupConfig,
+    cfg: NexusConfig,
     privs: ClientPrivateKeysFile,
     client: HttpClient,
-    orderType: KeysOrderType
+    order: EbicsKeyMng.Order
 ) {
-    logger.info("Doing key request ${orderType.name}")
-    val req = when(orderType) {
-        KeysOrderType.INI -> generateIniMessage(cfg, privs)
-        KeysOrderType.HIA -> generateHiaMessage(cfg, privs)
-        KeysOrderType.HPB -> generateHpbMessage(cfg, privs)
+    logger.info("Doing key request $order")
+    val req = EbicsKeyMng(cfg, privs, true).request(order)
+    val xml = client.postToBank(cfg.hostBaseUrl, req, order.name)
+    val resp = EbicsKeyMng.parseResponse(xml, privs.encryption_private_key)
+    
+    when (order) {
+        INI, HIA -> {
+            if (resp.technicalCode == EbicsReturnCode.EBICS_INVALID_USER_OR_USER_STATE) {
+                throw Exception("$order status code ${resp.technicalCode}: either your IDs are incorrect, or you already have keys registered with this bank")
+            }
+        }
+        HPB -> {
+            if (resp.technicalCode == EbicsReturnCode.EBICS_AUTHENTICATION_FAILED) {
+                throw Exception("$order status code ${resp.technicalCode}: could not download bank keys, send client keys (and/or related PDF document with --generate-registration-pdf) to the bank")
+            }
+        }
     }
-    val xml = try {
-        client.postToBank(cfg.hostBaseUrl, req)
-    } catch (e: Exception) {
-        throw Exception("Could not POST the ${orderType.name} message to the bank at '${cfg.hostBaseUrl}'", e)
+    
+    val orderData = resp.okOrFail(order.name)
+    when (order) {
+        INI -> privs.submitted_ini = true
+        HIA -> privs.submitted_hia = true
+        HPB -> {
+            val orderData = requireNotNull(orderData) {
+                "HPB: missing order data"
+            }
+            val (authPub, encPub) = EbicsKeyMng.parseHpbOrder(orderData)
+            val bankKeys = BankPublicKeysFile(
+                bank_authentication_public_key = authPub,
+                bank_encryption_public_key = encPub,
+                accepted = false
+            )
+            try {
+                persistBankKeys(bankKeys, cfg.bankPublicKeysPath)
+            } catch (e: Exception) {
+                throw Exception("Could not update the $order state on disk", e)
+            }
+        }
     }
-    val ebics = parseKeysMgmtResponse(privs.encryption_private_key, xml)
-    if (ebics == null) {
-        throw Exception("Could not get any EBICS from the bank ${orderType.name} response ($xml).")
-    }
-    if (ebics.technicalReturnCode != EbicsReturnCode.EBICS_OK) {
-        throw Exception("EBICS ${orderType.name} failed with code: ${ebics.technicalReturnCode}")
-    }
-    if (ebics.bankReturnCode != EbicsReturnCode.EBICS_OK) {
-        throw Exception("EBICS ${orderType.name} reached the bank, but could not be fulfilled, error code: ${ebics.bankReturnCode}")
-    }
-
-    when (orderType) {
-        KeysOrderType.INI -> privs.submitted_ini = true
-        KeysOrderType.HIA -> privs.submitted_hia = true
-        KeysOrderType.HPB -> return handleHpbResponse(cfg, ebics)
-    }
-    try {
-        persistClientKeys(privs, cfg.clientPrivateKeysFilename)
-    } catch (e: Exception) {
-        throw Exception("Could not update the ${orderType.name} state on disk", e)
+    if (order != HPB) {
+        try {
+            persistClientKeys(privs, cfg.clientPrivateKeysPath)
+        } catch (e: Exception) {
+            throw Exception("Could not update the $order state on disk", e)
+        }
     }
+    
 }
 
 /**
@@ -186,9 +155,9 @@ suspend fun doKeysRequestAndUpdateState(
  * @param configFile location of the configuration entry point.
  * @return internal representation of the configuration.
  */
-fun extractEbicsConfig(configFile: Path?): EbicsSetupConfig {
+fun extractEbicsConfig(configFile: Path?): NexusConfig {
     val config = loadConfig(configFile)
-    return EbicsSetupConfig(config)
+    return NexusConfig(config)
 }
 
 /**
@@ -198,7 +167,7 @@ fun extractEbicsConfig(configFile: Path?): EbicsSetupConfig {
  * @param privs client private keys.
  * @param cfg configuration handle.
  */
-private fun makePdf(privs: ClientPrivateKeysFile, cfg: EbicsSetupConfig) {
+private fun makePdf(privs: ClientPrivateKeysFile, cfg: NexusConfig) {
     val pdf = generateKeysPdf(privs, cfg)
     val path = Path("/tmp/libeufin-nexus-keys-${Instant.now().epochSecond}.pdf")
     try {
@@ -229,33 +198,37 @@ class EbicsSetup: CliktCommand("Set up the EBICS subscriber") {
      */
     override fun run() = cliCmd(logger, common.log) {
         val cfg = extractEbicsConfig(common.config)
-        // Config is sane.  Go (maybe) making the private keys.
-        val clientKeys = loadOrGenerateClientKeys(cfg.clientPrivateKeysFilename)
-        val httpClient = HttpClient()
+        val client =  HttpClient {
+            install(HttpTimeout) {
+                // It can take a lot of time for the bank to generate documents
+                socketTimeoutMillis = 5 * 60 * 1000
+            }
+        }
+
+        val clientKeys = loadOrGenerateClientKeys(cfg.clientPrivateKeysPath)
+        var bankKeys = loadBankKeys(cfg.bankPublicKeysPath)
+
+        // Check EBICS 3 support
+        val versions = HEV(client, cfg)
+        logger.debug("HEV: $versions")
+        if (!versions.contains(VersionNumber(3.0f, "H005")) && !versions.contains(VersionNumber(3.02f, "H005"))) {
+            throw Exception("EBICS 3 is not supported by your bank")
+        }
+
         // Privs exist.  Upload their pubs
         val keysNotSub = !clientKeys.submitted_ini
         if ((!clientKeys.submitted_ini) || forceKeysResubmission)
-            doKeysRequestAndUpdateState(cfg, clientKeys, httpClient, KeysOrderType.INI)
+            doKeysRequestAndUpdateState(cfg, clientKeys, client, INI)
         // Eject PDF if the keys were submitted for the first time, or the user asked.
         if (keysNotSub || generateRegistrationPdf) makePdf(clientKeys, cfg)
         if ((!clientKeys.submitted_hia) || forceKeysResubmission)
-            doKeysRequestAndUpdateState(cfg, clientKeys, httpClient, KeysOrderType.HIA)
+            doKeysRequestAndUpdateState(cfg, clientKeys, client, HIA)
         
-        // Checking if the bank keys exist on disk.
-        var bankKeys = loadBankKeys(cfg.bankPublicKeysFilename)
+        // Checking if the bank keys exist on disk
         if (bankKeys == null) {
-            try {
-                doKeysRequestAndUpdateState(
-                    cfg,
-                    clientKeys,
-                    httpClient,
-                    KeysOrderType.HPB
-                )
-            } catch (e: Exception) {
-                throw Exception("Could not download bank keys. Send client keys (and/or related PDF document with --generate-registration-pdf) to the bank", e)
-            }
-            logger.info("Bank keys stored at ${cfg.bankPublicKeysFilename}")
-            bankKeys = loadBankKeys(cfg.bankPublicKeysFilename)!!
+            doKeysRequestAndUpdateState(cfg, clientKeys, client, HPB)
+            logger.info("Bank keys stored at ${cfg.bankPublicKeysPath}")
+            bankKeys = loadBankKeys(cfg.bankPublicKeysPath)!!
         }
 
         if (!bankKeys.accepted) {
@@ -264,14 +237,36 @@ class EbicsSetup: CliktCommand("Set up the EBICS subscriber") {
             else bankKeys.accepted = askUserToAcceptKeys(bankKeys)
 
             if (!bankKeys.accepted) {
-                throw Exception("Cannot successfully finish the setup without accepting the bank keys.")
+                throw Exception("Cannot successfully finish the setup without accepting the bank keys")
             }
             try {
-                persistBankKeys(bankKeys, cfg.bankPublicKeysFilename)
+                persistBankKeys(bankKeys, cfg.bankPublicKeysPath)
             } catch (e: Exception) {
-                throw Exception("Could not set bank keys as accepted on disk.", e)
+                throw Exception("Could not set bank keys as accepted on disk", e)
             }
         }
+
+        // Check account information
+        logger.info("Doing administrative request HKD")
+        try {
+            ebicsDownload(client, cfg, clientKeys, bankKeys, EbicsOrder.V3("HKD"), null, null) { stream ->
+                val hkd = EbicsAdministrative.parseHKD(stream)
+                val account = hkd.account
+                // TODO parse and check more information
+                if (account.currency != null && account.currency != cfg.currency)
+                    logger.error("Expected CURRENCY '${cfg.currency}' from config got '${account.currency}' from bank")
+                if (account.iban != null && account.iban != cfg.account.iban)
+                    logger.error("Expected IBAN '${cfg.account.iban}' from config got '${account.iban}' from bank")
+                if (account.name != null && account.name != cfg.account.name)
+                    logger.warn("Expected NAME '${cfg.account.name}' from config got '${account.name}' from bank")
+
+                for (order in hkd.orders) {
+                    logger.debug("${order.type}${order.params}: ${order.description}")
+                }
+            }
+        } catch (e: Exception) {
+            logger.warn("HKD failed: ${e.fmt()}")
+        }
         
         println("setup ready")
     }
