@@ -19,7 +19,6 @@
 package tech.libeufin.nexus
 
 import tech.libeufin.common.*
-import tech.libeufin.ebics.*
 import java.io.InputStream
 import java.net.URLEncoder
 import java.time.*
@@ -76,14 +75,14 @@ fun createPain001(
     amount: TalerAmount,
     wireTransferSubject: String,
     creditAccount: IbanAccountMetadata
-): String {
+): ByteArray {
     val namespace = Pain001Namespaces(
         fullNamespace = "urn:iso:std:iso:20022:tech:xsd:pain.001.001.09",
         xsdFilename = "pain.001.001.09.ch.03.xsd"
     )
     val zonedTimestamp = ZonedDateTime.ofInstant(initiationTimestamp, ZoneId.of("UTC"))
     val amountWithoutCurrency: String = getAmountNoCurrency(amount)
-    return constructXml("Document") {
+    return XmlBuilder.toBytes("Document") {
         attr("xmlns", namespace.fullNamespace)
         attr("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance")
         attr("xsi:schemaLocation", "${namespace.fullNamespace} ${namespace.xsdFilename}")
@@ -144,13 +143,9 @@ data class CustomerAck(
     }
 }
 
-/**
- * Extract logs from a pain.002 HAC document.
- *
- * @param xml pain.002 input document
- */
+/** Parse HAC pain.002 XML file */
 fun parseCustomerAck(xml: InputStream): List<CustomerAck> {
-    return destructXml(xml, "Document") {
+    return XmlDestructor.fromStream(xml, "Document") {
         one("CstmrPmtStsRpt").map("OrgnlPmtInfAndSts") {
             val actionType = one("OrgnlPmtInfId").enum<HacAction>()
             one("StsRsnInf") {
@@ -193,19 +188,21 @@ data class PaymentStatus(
 
     fun description(): String = txCode?.description ?: paymentCode.description
 
-    override fun toString(): String {
+    fun msg(): String {
         return if (reasons.isEmpty()) {
-            "'${id()}' ${code()} '${description()}'"
+            "${code()} '${description()}'"
         } else if (reasons.size == 1) {
-            "'${id()}' ${code()} ${reasons[0].code.isoCode} - '${description()}' '${reasons[0].code.description}'"
+            "${code()} ${reasons[0].code.isoCode} - '${description()}' '${reasons[0].code.description}'"
         } else {
-            var str = "'${id()}' ${code()} '${description()}' - "
+            var str = "${code()} '${description()}' - "
             for (reason in reasons) {
-                str += "${reason.code.isoCode} '${reason.code.description}'"
+                str += "${reason.code.isoCode} '${reason.code.description}' "
             }
             str
         }
     }
+
+    override fun toString(): String = "${id()} ${msg()}"
 }
 
 data class Reason (
@@ -213,11 +210,7 @@ data class Reason (
     val information: String
 )
 
-/**
- * Extract payment status from a pain.002 document.
- *
- * @param xml pain.002 input document
- */
+/** Parse pain.002 XML file */
 fun parseCustomerPaymentStatusReport(xml: InputStream): PaymentStatus {
     fun XmlDestructor.reasons(): List<Reason> {
         return map("StsRsnInf") {
@@ -226,7 +219,7 @@ fun parseCustomerPaymentStatusReport(xml: InputStream): PaymentStatus {
             Reason(code, "")
         }
     }
-    return destructXml(xml, "Document") {
+    return XmlDestructor.fromStream(xml, "Document") {
         // TODO handle batch status
         one("CstmrPmtStsRpt") {
             val (msgId, msgCode, msgReasons) = one("OrgnlGrpInfAndSts") {
@@ -251,30 +244,76 @@ fun parseCustomerPaymentStatusReport(xml: InputStream): PaymentStatus {
 }
 
 sealed interface TxNotification {
-    data class Incoming(val payment: IncomingPayment): TxNotification
-    data class Outgoing(val payment: OutgoingPayment): TxNotification
-    data class Reversal(val reversal: OutgoingReversal): TxNotification
+    val executionTime: Instant
+    data class Reversal(
+        val msgId: String,
+        val reason: String?,
+        override val executionTime: Instant
+    ): TxNotification
 }
 
-data class OutgoingReversal(
-    val bankId: String,
-    val reason: String?
-)
+/** ISO20022 incoming payment */
+data class IncomingPayment(
+    val amount: TalerAmount,
+    val wireTransferSubject: String,
+    val debitPaytoUri: String,
+    override val executionTime: Instant,
+    /** ISO20022 AccountServicerReference */
+    val bankId: String
+): TxNotification {
+    override fun toString(): String {
+        return "IN ${executionTime.fmtDate()} $amount '$bankId' debitor=$debitPaytoUri subject=$wireTransferSubject"
+    }
+}
 
-/**
- * Searches payments in a camt.054 (Detailavisierung) document.
- *
- * @param notifXml camt.054 input document
- * @param acceptedCurrency currency accepted by Nexus
- * @param incoming list of incoming payments
- * @param outgoing list of outgoing payments
- */
+/** ISO20022 outgoing payment */
+data class OutgoingPayment(
+    val amount: TalerAmount,
+    override val executionTime: Instant,
+    /** ISO20022 MessageIdentification */
+    val messageId: String,
+    val creditPaytoUri: String? = null, // not showing in camt.054
+    val wireTransferSubject: String? = null // not showing in camt.054
+): TxNotification {
+    override fun toString(): String {
+        return "OUT ${executionTime.fmtDate()} $amount '$messageId' creditor=$creditPaytoUri subject=$wireTransferSubject"
+    }
+}
+
+/** Parse camt.054 XML file */
 fun parseTxNotif(
     notifXml: InputStream,
-    acceptedCurrency: String,
-    notifications: MutableList<TxNotification>,
-) {
-    notificationForEachTx(notifXml) { bookDate, reversal, info ->
+    acceptedCurrency: String
+): List<TxNotification> {
+    fun notificationForEachTx(
+        directionLambda: XmlDestructor.(Instant, Boolean, String?) -> Unit
+    ) {
+        XmlDestructor.fromStream(notifXml, "Document") {
+            opt("BkToCstmrDbtCdtNtfctn")?.each("Ntfctn") {
+                each("Ntry") {
+                    val reversal = opt("RvslInd")?.bool() ?: false
+                    val info = opt("AddtlNtryInf")?.text()
+                    one("Sts") {
+                        if (text() != "BOOK") {
+                            one("Cd") {
+                                if (text() != "BOOK")
+                                    throw Exception("Found non booked transaction, " +
+                                            "stop parsing.  Status was: ${text()}"
+                                    )
+                            }
+                        }
+                    }
+                    val bookDate: Instant = one("BookgDt").one("Dt").date().atStartOfDay().toInstant(ZoneOffset.UTC)
+                    one("NtryDtls").each("TxDtls") {
+                        directionLambda(this, bookDate, reversal, info)
+                    }
+                }
+            }
+        }
+    }
+
+    val notifications = mutableListOf<TxNotification>()
+    notificationForEachTx { bookDate, reversal, info ->
         val kind = one("CdtDbtInd").text()
         val amount: TalerAmount = one("Amt") {
             val currency = attr("Ccy")
@@ -290,10 +329,11 @@ fun parseTxNotif(
             if (msgId == null) {
                 logger.debug("Unsupported reversal without message id")
             } else {
-                notifications.add(TxNotification.Reversal(OutgoingReversal(
-                    bankId = msgId,
-                    reason = info
-                )))
+                notifications.add(TxNotification.Reversal(
+                    msgId = msgId,
+                    reason = info,
+                    executionTime = bookDate
+                ))
             }
             return@notificationForEachTx
         }
@@ -319,61 +359,24 @@ fun parseTxNotif(
                         debtorPayto.append("?receiver-name=$urlEncName")
                     }
                 }
-                notifications.add(TxNotification.Incoming(
-                    IncomingPayment(
-                        amount = amount,
-                        bankId = bankId,
-                        debitPaytoUri = debtorPayto.toString(),
-                        executionTime = bookDate,
-                        wireTransferSubject = subject.toString()
-                    )
+                notifications.add(IncomingPayment(
+                    amount = amount,
+                    bankId = bankId,
+                    debitPaytoUri = debtorPayto.toString(),
+                    executionTime = bookDate,
+                    wireTransferSubject = subject.toString()
                 ))
             }
             "DBIT" -> {
                 val messageId = one("Refs").one("MsgId").text()
-                notifications.add(TxNotification.Outgoing(
-                    OutgoingPayment(
-                        amount = amount,
-                        messageId = messageId,
-                        executionTime = bookDate
-                    )
+                notifications.add(OutgoingPayment(
+                    amount = amount,
+                    messageId = messageId,
+                    executionTime = bookDate
                 ))
             }
             else -> throw Exception("Unknown transaction notification kind '$kind'")
         }        
     }
-}
-
-/**
- * Navigates the camt.054 (Detailavisierung) until its leaves, where
- * then it invokes the related parser, according to the payment direction.
- *
- * @param xml the input document.
- */
-private fun notificationForEachTx(
-    xml: InputStream,
-    directionLambda: XmlDestructor.(Instant, Boolean, String?) -> Unit
-) {
-    destructXml(xml, "Document") {
-        opt("BkToCstmrDbtCdtNtfctn")?.each("Ntfctn") {
-            each("Ntry") {
-                val reversal = opt("RvslInd")?.bool() ?: false
-                val info = opt("AddtlNtryInf")?.text()
-                one("Sts") {
-                    if (text() != "BOOK") {
-                        one("Cd") {
-                            if (text() != "BOOK")
-                                throw Exception("Found non booked transaction, " +
-                                        "stop parsing.  Status was: ${text()}"
-                                )
-                        }
-                    }
-                }
-                val bookDate: Instant = one("BookgDt").one("Dt").date().atStartOfDay().toInstant(ZoneOffset.UTC)
-                one("NtryDtls").each("TxDtls") {
-                    directionLambda(this, bookDate, reversal, info)
-                }
-            }
-        }
-    }
+    return notifications
 }
\ No newline at end of file
