@@ -27,20 +27,32 @@ package tech.libeufin.nexus
 
 import com.github.ajalt.clikt.core.CliktCommand
 import com.github.ajalt.clikt.core.subcommands
-import com.github.ajalt.clikt.parameters.arguments.*
-import com.github.ajalt.clikt.parameters.groups.*
+import com.github.ajalt.clikt.parameters.arguments.argument
+import com.github.ajalt.clikt.parameters.arguments.convert
+import com.github.ajalt.clikt.parameters.groups.provideDelegate
 import com.github.ajalt.clikt.parameters.options.*
-import io.ktor.client.*
-import io.ktor.util.*
-import kotlinx.coroutines.*
+import com.github.ajalt.clikt.parameters.types.path
+import kotlinx.serialization.json.Json
+import kotlinx.serialization.Serializable
+import kotlin.io.path.*
+import io.ktor.server.application.*
 import org.slf4j.Logger
+import org.slf4j.event.Level
 import org.slf4j.LoggerFactory
 import tech.libeufin.common.*
-import tech.libeufin.nexus.ebics.*
+import tech.libeufin.common.api.*
+import tech.libeufin.common.crypto.*
+import tech.libeufin.common.db.DatabaseConfig
+import tech.libeufin.nexus.api.*
+import tech.libeufin.nexus.db.Database
+import tech.libeufin.nexus.db.InitiatedPayment
 import java.nio.file.Path
 import java.time.Instant
+import java.time.ZoneId
+import java.time.format.DateTimeFormatter
+import javax.crypto.EncryptedPrivateKeyInfo
+
 
-val NEXUS_CONFIG_SOURCE = ConfigSource("libeufin", "libeufin-nexus", "libeufin-nexus")
 internal val logger: Logger = LoggerFactory.getLogger("libeufin-nexus")
 
 /**
@@ -52,149 +64,17 @@ data class IbanAccountMetadata(
     val name: String
 )
 
-/**
- * Contains the frequency of submit or fetch iterations.
- */
-data class NexusFrequency(
-    /**
-     * Value in seconds of the FREQUENCY configuration
-     * value, found either under [nexus-fetch] or [nexus-submit]
-     */
-    val inSeconds: Int,
-    /**
-     * Copy of the value found in the configuration.  Used
-     * for logging.
-     */
-    val fromConfig: String
-)
+fun Instant.fmtDate(): String = 
+    DateTimeFormatter.ISO_LOCAL_DATE.withZone(ZoneId.of("UTC")).format(this)
 
-/**
- * Converts human-readable duration in how many seconds.  Supports
- * the suffixes 's' (seconds), 'm' (minute), 'h' (hours).  A valid
- * duration is therefore, for example, Nm, where N is the number of
- * minutes.
- *
- * @param trimmed duration
- * @return how many seconds is the duration input, or null if the input
- *         is not valid.
- */
-fun getFrequencyInSeconds(humanFormat: String): Int? {
-    val trimmed = humanFormat.trim()
-    if (trimmed.isEmpty()) {
-        logger.error("Input was empty")
-        return null
-    }
-    val howManySeconds: Int = when (val lastChar = trimmed.last()) {
-        's' -> {1}
-        'm' -> {60}
-        'h' -> {60 * 60}
-        else -> {
-            logger.error("Duration symbol not one of s, m, h.  '$lastChar' was found instead")
-            return null
-        }
-    }
-    val maybeNumber = trimmed.dropLast(1)
-    val howMany = try {
-        maybeNumber.trimEnd().toInt()
-    } catch (e: Exception) {
-        logger.error("Prefix was not a valid input: '$maybeNumber'")
-        return null
-    }
-    if (howMany == 0) return 0
-    val ret = howMany * howManySeconds
-    if (howMany != ret / howManySeconds) {
-        logger.error("Result overflew")
-        return null
-    }
-    return ret
+fun Instant.fmtDateTime(): String =
+    DateTimeFormatter.ISO_LOCAL_DATE_TIME.withZone(ZoneId.of("UTC")).format(this)
+
+
+fun Application.nexusApi(db: Database, cfg: NexusConfig) = talerApi(logger) {
+    wireGatewayApi(db, cfg)
 }
 
-/**
- * Sanity-checks the frequency found in the configuration and
- * either returns it or fails the process.  Note: the returned
- * value is also guaranteed to be non-negative.
- *
- * @param foundInConfig frequency value as found in the configuration.
- * @return the duration in seconds of the value found in the configuration.
- */
-fun checkFrequency(foundInConfig: String): Int {
-    val frequencySeconds = getFrequencyInSeconds(foundInConfig)
-        ?: throw Exception("Invalid frequency value in config section nexus-submit: $foundInConfig")
-    if (frequencySeconds < 0) {
-        throw Exception("Configuration error: cannot operate with a negative submit frequency ($foundInConfig)")
-    }
-    return frequencySeconds
-}
-
-
-/**
- * Keeps all the options of the ebics-setup subcommand.  The
- * caller has to handle TalerConfigError if values are missing.
- * If even one of the fields could not be instantiated, then
- * throws TalerConfigError.
- */
-class EbicsSetupConfig(val config: TalerConfig) {
-    // abstracts the section name.
-    private val ebicsSetupRequireString = { option: String ->
-        config.requireString("nexus-ebics", option)
-    }
-    private val ebicsSetupRequirePath = { option: String ->
-        config.requirePath("nexus-ebics", option)
-    }
-    // debug utility to inspect what was loaded.
-    fun _dump() {
-        this.javaClass.declaredFields.forEach {
-            println("cfg obj: ${it.name} -> ${it.get(this)}")
-        }
-    }
-    /**
-     * The bank's currency.
-     */
-    val currency = ebicsSetupRequireString("currency")
-    /**
-     * The bank base URL.
-     */
-    val hostBaseUrl = ebicsSetupRequireString("host_base_url")
-    /**
-     * The bank EBICS host ID.
-     */
-    val ebicsHostId = ebicsSetupRequireString("host_id")
-    /**
-     * EBICS user ID.
-     */
-    val ebicsUserId = ebicsSetupRequireString("user_id")
-    /**
-     * EBICS partner ID.
-     */
-    val ebicsPartnerId = ebicsSetupRequireString("partner_id")
-    /**
-     * Bank account metadata.
-     */
-    val myIbanAccount = IbanAccountMetadata(
-        iban = ebicsSetupRequireString("iban"),
-        bic = ebicsSetupRequireString("bic"),
-        name = ebicsSetupRequireString("name")
-    )
-    /**
-     * Filename where we store the bank public keys.
-     */
-    val bankPublicKeysFilename = ebicsSetupRequirePath("bank_public_keys_file")
-    /**
-     * Filename where we store our private keys.
-     */
-    val clientPrivateKeysFilename = ebicsSetupRequirePath("client_private_keys_file")
-    /**
-     * A name that identifies the EBICS and ISO20022 flavour
-     * that Nexus should honor in the communication with the
-     * bank.
-     */
-    val bankDialect: String = ebicsSetupRequireString("bank_dialect").run {
-        if (this != "postfinance") throw Exception("Only 'postfinance' dialect is supported.")
-        return@run this
-    }
-}
-
-
 /**
  * Abstracts the config loading
  *
@@ -250,8 +130,8 @@ class InitiatePayment: CliktCommand("Initiate an outgoing payment") {
             Base32Crockford.encode(bytes)
         }
 
-        Database(dbCfg.dbConnStr).use { db ->
-            db.initiatedPaymentCreate(
+        Database(dbCfg).use { db ->
+            db.initiated.create(
                 InitiatedPayment(
                     id = -1,
                     amount = amount,
@@ -265,6 +145,103 @@ class InitiatePayment: CliktCommand("Initiate an outgoing payment") {
     }
 }
 
+class ConvertBackup: CliktCommand("Convert an old backup to the new config format") {
+    private val backupPath by argument(
+        "backup",
+        help = "Specifies the backup file"
+    ).path()
+
+    @Serializable
+    data class EbicsKeysBackupJson(
+        val userID: String,
+        val partnerID: String,
+        val hostID: String,
+        val ebicsURL: String,
+        val authBlob: String,
+        val encBlob: String,
+        val sigBlob: String
+    )
+
+    override fun run() = cliCmd(logger, Level.INFO) {
+        val raw = backupPath.readText()
+        val backup = Json.decodeFromString<EbicsKeysBackupJson>(raw)
+
+        val (authBlob, encBlob, sigBlob) = Triple(
+            EncryptedPrivateKeyInfo(backup.authBlob.decodeBase64()),
+            EncryptedPrivateKeyInfo(backup.encBlob.decodeBase64()),
+            EncryptedPrivateKeyInfo(backup.sigBlob.decodeBase64())
+        )
+        lateinit var keys: ClientPrivateKeysFile
+        while (true) {
+            val passphrase = prompt("Enter the backup password", hideInput = true)!!
+            try {
+                val (authKey, encKey, sigKey) = Triple(
+                    CryptoUtil.decryptKey(authBlob, passphrase),
+                    CryptoUtil.decryptKey(encBlob, passphrase),
+                    CryptoUtil.decryptKey(sigBlob, passphrase)
+                )
+                keys = ClientPrivateKeysFile(
+                    signature_private_key = sigKey,
+                    encryption_private_key = encKey,
+                    authentication_private_key = authKey,
+                    submitted_ini = false,
+                    submitted_hia = false
+                )
+                break
+            } catch (e: Exception) {
+                e.fmtLog(logger)
+            }
+        }
+       
+
+        println("# KEYS")
+        println(JSON.encodeToString(kotlinx.serialization.serializer<ClientPrivateKeysFile>(), keys))
+
+        println("# CONFIG")
+        println("""
+[nexus-ebics]
+CURRENCY = CHF
+
+HOST_BASE_URL = ${backup.ebicsURL}
+BANK_DIALECT = postfinance
+
+
+HOST_ID = ${backup.hostID}
+USER_ID = ${backup.userID}
+PARTNER_ID = ${backup.partnerID}
+SYSTEM_ID = 
+
+IBAN = 
+BIC = 
+NAME = 
+""")
+
+        /*val (authKey, encKey, sigKey) = try {
+            Triple(
+                CryptoUtil.decryptKey(
+                    EncryptedPrivateKeyInfo(base64ToBytes(ebicsBackup.authBlob)),
+                    passphrase
+                ),
+                CryptoUtil.decryptKey(
+                    EncryptedPrivateKeyInfo(base64ToBytes(ebicsBackup.encBlob)),
+                    passphrase
+                ),
+                CryptoUtil.decryptKey(
+                    EncryptedPrivateKeyInfo(base64ToBytes(ebicsBackup.sigBlob)),
+                    passphrase
+                )
+            )
+        } catch (e: Exception) {
+            e.printStackTrace()
+            logger.info("Restoring keys failed, probably due to wrong passphrase")
+            throw NexusError(
+                HttpStatusCode.BadRequest,
+                "Bad backup given"
+            )
+        }*/
+    }
+}
+
 class FakeIncoming: CliktCommand("Genere a fake incoming payment") {
     private val common by CommonOption()
     private val amount by option(
@@ -296,7 +273,7 @@ class FakeIncoming: CliktCommand("Genere a fake incoming payment") {
             Base32Crockford.encode(bytes)
         }
 
-        Database(dbCfg.dbConnStr).use { db ->
+        Database(dbCfg).use { db ->
             ingestIncomingPayment(db, 
                 IncomingPayment(
                     amount = amount,
@@ -312,7 +289,7 @@ class FakeIncoming: CliktCommand("Genere a fake incoming payment") {
 
 class TestingCmd : CliktCommand("Testing helper commands", name = "testing") {
     init {
-        subcommands(FakeIncoming())
+        subcommands(FakeIncoming(), ConvertBackup())
     }
 
     override fun run() = Unit
